; ========================================================================================================-
; Chipmunk2D SDK by Lunasole [2015/11]
; http://geocities.ws/lunasole
; ==========================================================================================================================-
; Thanks to IceSoft, author of Purebasic bindings to Chipmunk v5.3.5 for help with 1st working version of this
; ============================================================================================================================================-
; Chipmunk2D lib version: 	7.0.1
; PureBasic version: 		5.40+
; SDK version:				0.0.40
; ============================-

EnableExplicit

; ========================================================================================================
;	Some basic Chipmunk2D Pseudotypes
; ========================================================================================================
; Chipmunk :: Basic Types. The macro here used to represent them for handy usage

Macro cpFloat			: d : EndMacro ; double.		Depends on chipmunk build: double if ifndef CP_USE_DOUBLES, else it uses regular Float
Macro cpBool 			: l : EndMacro ; int.			Like every good C library that wants good cross language compatibility, you end up defining your own boolean type. :-\ Defaults to int.

Macro cpCollisionType 	: i : EndMacro ; unsigned int.	Unique identifier for collision shape types. Defaults to unsigned int. Defined type must support the == operator.
Macro cpGroup 			: i : EndMacro ; uintptr_t.		Unique identifier for collision groups. Defaults to unsigned int. A CP_NO_GROUP value is defined that can be used when you dont want to specify a group. Defined type must support the equality == operator.
Macro cpBitmask 		: l : EndMacro ; unsigned int.	Type used for collision filter categories and masks. Defaults to unsigned int. Defined type must support the bitwise AND & operator.

; Some others pseudotypes
Macro cpCollisionID 	: l : EndMacro ; unsigned int
Macro cpTimestamp 		: l : EndMacro ; unsigned int

Macro cpHashValue 		: i : EndMacro ; uintptr_t

; Enums. The macro here are used as pseutotypes which can take values of corresponding enum
Macro cpShapeType 		: l : EndMacro
	Enumeration cpShapeType_e
		#CP_CIRCLE_SHAPE
		#CP_SEGMENT_SHAPE
		#CP_POLY_SHAPE
		#CP_NUM_SHAPES
	EndEnumeration
	
Macro cpArbiterState 	: l : EndMacro
	Enumeration cpArbiterState_e
		; Arbiter is active And its the first collision.
		#CP_ARBITER_STATE_FIRST_COLLISION
		; Arbiter is active And its Not the first collision.
		#CP_ARBITER_STATE_NORMAL
		; Collision has been explicitly ignored.
		; Either by returning false from a begin collision handler Or calling cpArbiterIgnore().
		#CP_ARBITER_STATE_IGNORE
		; Collison is no longer active. A space will cache an arbiter For up To cpSpace.collisionPersistence more steps.
		#CP_ARBITER_STATE_CACHED
		; Collison arbiter is invalid because one of the shapes was removed.
		#CP_ARBITER_STATE_INVALIDATED
	EndEnumeration

Macro cpBodyType 		: l : EndMacro
	Enumeration cpBodyType_e
		; A dynamic body is one that is affected by gravity, forces, And collisions.
		; This is the Default body type.
		#CP_BODY_TYPE_DYNAMIC
		; A kinematic body is an infinite mass, user controlled body that is Not affected by gravity, forces Or collisions.
		; Instead the body only moves based on it's velocity.
		; Dynamic bodies collide normally With kinematic bodies, though the kinematic body will be unaffected.
		; Collisions between two kinematic bodies, Or a kinematic body And a Static body produce collision callbacks, but no collision response.
		#CP_BODY_TYPE_KINEMATIC
		; A Static body is a body that never (Or rarely) moves. If you move a Static body, you must call one of the cpSpaceReindex*() functions.
		; Chipmunk uses this information To optimize the collision detection.
		; Static bodies do Not produce collision callbacks when colliding With other Static bodies.
		#CP_BODY_TYPE_STATIC
	EndEnumeration


; ========================================================================================================
;	These should not be changed
; ========================================================================================================

	#CP_MAX_CONTACTS_PER_ARBITER 	= 2	; Size of the Array of contact points
	#CP_POLY_SHAPE_INLINE_ALLOC 	= 6 ; Size of cpPolyShape\_planes array (const * 2)

; ========================================================================================================
;	Structures
; ========================================================================================================
; Note: some structure fields here are renamed to be more clear: cpShape\u >> cpShape\friction, etc
; 	In such cases there are related marks showing original name inside comment
; ========================================================================================================
	; ========================================================================================================
	; [unclassified yet]
	; ========================================================================================================

	Structure cpArray
		num.l
		max.l
	 	*arr
	EndStructure
	
	Structure cpVect ; Chipmunk's 2D vector type.
		X.cpFloat
		Y.cpFloat
	EndStructure

	
	Structure cpTransform
		a.cpFloat
		b.cpFloat
		c.cpFloat
		d.cpFloat
		tx.cpFloat
		ty.cpFloat
	EndStructure
	
	Structure cpSpatialIndexClass
		*cpSpatialIndexDestroyImpl ; destroy
		
		*cpSpatialIndexCountImpl ; count
		*cpSpatialIndexEachImpl ; each
		
		*cpSpatialIndexContainsImpl ; contains
		*cpSpatialIndexInsertImpl ; insert
		*cpSpatialIndexRemoveImpl ; remove
		
		*cpSpatialIndexReindexImpl ; reindex
		*cpSpatialIndexReindexObjectImpl ; reindexObject
		*cpSpatialIndexReindexQueryImpl ; reindexQuery
		
		*cpSpatialIndexQueryImpl ; query
		*cpSpatialIndexSegmentQueryImpl ; segmentQuery
	EndStructure
	
	Structure cpSpatialIndex
		*klass.cpSpatialIndexClass 
		
		*cpSpatialIndexBBFunc ;  bbfunc
		
		*dynamicIndex.cpSpatialIndex 
		*staticIndex.cpSpatialIndex 
	EndStructure
	
	Structure cpBB  ; Chipmunk's axis-aligned 2D bounding box type. (left, bottom, right, top)
		L.cpFloat
		B.cpFloat 
		R.cpFloat 
		T.cpFloat
	EndStructure
	
	Structure cpBBTree
		spatialIndex.cpSpatialIndex
		*cpBBTreeVelocityFunc ;velocityFunc
		
		*leaves.cpHashSet
		*root.node
		
		*pooledNodes.node
		*pooledPairs.pair
		*allocatedBuffers.cpArray
		stamp.cpTimestamp
	EndStructure
	
	
	Structure cpMat2x2 ; Row major [[a, b][c d]]
		a.cpFloat : b.cpFloat : c.cpFloat : d.cpFloat 
	EndStructure
	
	Structure cpSplittingPlane
		v0.cpVect : n.cpVect
	EndStructure
	
	Structure cpHashSetBin
		*elt
		hash.cpHashValue
		*next.cpHashSetBin
	EndStructure
	
	Structure cpHashSet
		entries.l : size.l ; unsigned int
		
		*cpHashSetEqlFunc ; eql.
		*default_value
		
		*table.cpHashSetBin
		*pooledBins.cpHashSetBin
		
		*allocatedBuffers.cpArray
	EndStructure
	
	
	Structure Bounds
		min.cpFloat : max.cpFloat
	EndStructure
	
	Structure TableCell
		*obj
		bounds.Bounds
	EndStructure
	
	Structure cpSweep1D
		spatialIndex.cpSpatialIndex
		
		num.l
		max.l
		*table.TableCell
	EndStructure

	; ========================================================================================================
	; Rigid Bodies: A rigid body holds the physical properties of an object. (mass, position, rotation, velocity, etc.)
	; It does not have a shape until you attach one or more collision shapes to it. If youve done physics with particles before,
	; rigid bodies differ in that they are able to rotate. Rigid bodies generally tend to have a 1:1 correlation to sprites in a game.
	; You should structure your game so that you use the position and rotation of the rigid body for drawing your sprite.
	; ========================================================================================================
	
	Structure sleeping 
		*root.cpBody
		*next.cpBody
		idleTime.cpFloat
	EndStructure
	
	Structure cpBody
		; Integration functions
		*cpBodyVelocityFunc; velocity_func.
		*cpBodyPositionFunc; position_func.
		
		; mass And it's inverse
		mass.cpFloat				; [m] Mass of the rigid body. Mass does not have to be expressed in any particular units, but relative masses should be consistent.
		mass_inv.cpFloat			; [m_inv] Mass inverse.
		
		; moment of inertia And it's inverse
		inertia.cpFloat				; [i]
		inertia_inv.cpFloat			; [i_inv]
		
		; center of gravity
		centerOfGravity.cpVect		; [cog] Location of the body's center of gravity relative to it's position. Defaults to cpvzero.
		
		; position, velocity, force
		pos.cpVect					; [p] The position of the rigid body's center of gravity.
		vel.cpVect					; [v] The linear velocity of the rigid body.
		force.cpVect				; [f] The linear force applied to the rigid body. Unlike in some physics engines, the force does not reset itself during each step. Make sure that you are reseting the force between frames if that is what you intended.
		
		; Angle, angular velocity, torque (radians)
		angle.cpFloat				; [a] The rotation angle of the rigid body in radians.
		angle_vel.cpFloat			; [w] The angular velocity of the rigid body in radians per second.
		torque.cpFloat				; [t] The torque being applied to the rigid body. Like force, this property is not reset every frame.
		
		transform.cpTransform		; The rigid transform of the body.
		
		*userData					; An object that this constraint is associated with
		
		; "pseudo-velocities" used For eliminating overlap.
		; Erin Catto has some papers that talk about what these are.
		v_bias.cpVect
		w_bias.cpFloat
		
		*space.cpSpace
		
		*shapeList.cpShape
		*arbiterList.cpArbiter
		*constraintList.cpConstraint
		
 		sleeping.sleeping
	EndStructure

	
	; ========================================================================================================
	; Collision Shapes: By attaching shapes to bodies, you can define the a bodys shape. 
	; You can attach as many shapes to a single body as you need to in order to define a complex shape. 
	; Shapes contain the surface properties of an object such as how much friction or elasticity it has.
	; ========================================================================================================

	Structure cpShapeFilter ; Fast collision filtering type that is used To determine If two objects collide before calling collision Or query callbacks.
		; Two objects With the same non-zero group value do Not collide.
		; This is generally used To group objects in a composite object together To disable self collisions.
		group.cpGroup
		; A bitmask of user definable categories that this object belongs To.
		; The category/mask combinations of both objects in a collision must agree For a collision To occur.
		cat_belongs_to.cpBitmask
		; A bitmask of user definable category types that this object object collides With.
		; The category/mask combinations of both objects in a collision must agree For a collision To occur.
		cat_collide_with.cpBitmask
	EndStructure
	
	Structure cpShapeMassInfo
		 m.cpFloat
		 i.cpFloat
		 cog.cpVect
		 area.cpFloat
	EndStructure
	
	Structure cpShapeClass
		type.cpShapeType
		
		*cpShapeCacheDataImpl 	; cacheData
		*cpShapeDestroyImpl 	; destroy
		*cpShapePointQueryImpl 	; pointQuery
		*cpShapeSegmentQueryImpl ; *segmentQuery
	EndStructure
	
	Structure cpShape
		*klass.cpShapeClass
		
		*space.cpSpace
		*body.cpBody			; The rigid body this collision shape is attached to.
		massInfo.cpShapeMassInfo
		bb.cpBB					; The current bounding box of the shape.
		
		sensor.cpBool			; Sensor flag. Sensor shapes call collision callbacks but don't produce collisions.
		
		elasticity.cpFloat		; [e] Coefficient of restitution. (elasticity)
		friction.cpFloat		; [u] Coefficient of friction
		surfaceV.cpVect			; Surface velocity used when solving for friction.

		*userData				; User definable data pointer. Generally this points to your the game object class so you can access it when given a cpShape reference in a callback.
		
		type.cpCollisionType	; Collision type of this shape used when picking collision handlers.
		filter.cpShapeFilter
		
		*next.cpShape
		*prev.cpShape
		
		hashid.cpHashValue
	EndStructure
	
	Structure cpPolyShape
		shape.cpShape
		
		r.cpFloat
		
		count.l
		; The untransformed planes are appended at the End of the transformed planes.
		*planes.cpSplittingPlane
		
		; Allocate a small number of splitting planes internally For simple poly.
		_planes.cpSplittingPlane [2 * #CP_POLY_SHAPE_INLINE_ALLOC]
	EndStructure
	
	Structure cpCircleShape
		shape.cpShape
		
		c.cpVect : tc.cpVect
		r.cpFloat
	EndStructure
	
	Structure cpSegmentShape
		shape.cpShape
		
		a.cpVect : b.cpVect : n.cpVect
		ta.cpVect : tb.cpVect : tn.cpVect
		r.cpFloat
		
		a_tangent.cpVect : b_tangent.cpVect
	EndStructure
	
	Structure cpCollisionHandler; Struct that holds function callback pointers To configure custom collision handling. Collision handlers have a pair of types; when a collision occurs between two shapes that have these types, the collision handler functions are triggered.
		; Collision type identifier of the first shape that this handler recognizes.
		; In the collision handler callback, the shape With this type will be the first argument. Read only.
		typeA.cpCollisionType
		; Collision type identifier of the second shape that this handler recognizes.
		; In the collision handler callback, the shape With this type will be the second argument. Read only.
		typeB.cpCollisionType
		; This function is called when two shapes With types that match this collision handler begin colliding.
		*cpCollisionBeginFunc; beginFunc;
		; This function is called each Step when two shapes With types that match this collision handler are colliding.
		; It's called before the collision solver runs so that you can affect a collision's outcome.
		*cpCollisionPreSolveFunc; preSolveFunc;
		; This function is called each Step when two shapes With types that match this collision handler are colliding.
		; It's called after the collision solver runs so that you can read back information about the collision to trigger events in your game.
		*cpCollisionPostSolveFunc ;postSolveFunc;
		; This function is called when two shapes With types that match this collision handler stop colliding.
		*cpCollisionSeparateFunc; separateFunc;
		; This is a user definable context pointer that is passed To all of the collision handler functions.
		*userData ;cpDataPointer
	EndStructure
	
	Structure cpCollisionInfo
		*a.cpShape
		*b.cpShape
		id.cpCollisionID
		
		n.cpVect
		
		count.l
		*arr.cpContact
	EndStructure

	; ========================================================================================================
	; Constraints/Joints: Constraints and joints describe how bodies are attached to each other.
	; ========================================================================================================

	Structure cpConstraint
		*klass.cpConstraintClass
		
		*space.cpSpace
		
		*a.cpBody
		*b.cpBody
		*next_a.cpConstraint
		*next_b.cpConstraint
		
		maxForce.cpFloat
		errorBias.cpFloat
		maxBias.cpFloat
		
		collideBodies.cpBool
		
		*cpConstraintPreSolveFunc ; preSolve
		*cpConstraintPostSolveFunc ; postSolve
		
		*userData ;.cpDataPointer
	EndStructure
	
	Structure cpConstraintClass
		*cpConstraintPreStepImpl; preStep;
		*cpConstraintApplyCachedImpulseImpl ; applyCachedImpulse;
		*cpConstraintApplyImpulseImpl ;applyImpulse;
		*cpConstraintGetImpulseImpl ; getImpulse;
	EndStructure
	
	Structure cpGrooveJoint
		constraint.cpConstraint
		grv_n.cpVect : grv_a.cpVect : grv_b.cpVect
		anchorB.cpVect
		
		grv_tn.cpVect
		clamp.cpFloat
		r1.cpVect : r2.cpVect
		k.cpMat2x2
		
		jAcc.cpVect
		bias.cpVect
	EndStructure
	
	Structure cpPinJoint
		constraint.cpConstraint
		anchorA.cpVect : anchorB.cpVect
		dist.cpFloat
		
		r1.cpVect : r2.cpVect
		n.cpVect
		nMass.cpFloat
		
		jnAcc.cpFloat
		bias.cpFloat
	EndStructure
	
	Structure cpPivotJoint
		constraint.cpConstraint
		anchorA.cpVect : anchorB.cpVect
		
		r1.cpVect : r2.cpVect
		k.cpMat2x2
		
		jAcc.cpVect
		bias.cpVect
	EndStructure
	
	Structure cpRotaryLimitJoint
		constraint.cpConstraint
		min.cpFloat : max.cpFloat
		
		iSum.cpFloat
		
		bias.cpFloat
		jAcc.cpFloat
	EndStructure
	
	Structure cpRatchetJoint
		constraint.cpConstraint
		angle.cpFloat : phase.cpFloat : ratchet.cpFloat
		
		iSum.cpFloat
		
		bias.cpFloat
		jAcc.cpFloat
	EndStructure
	
	Structure cpGearJoint
		constraint.cpConstraint
		phase.cpFloat : ratio.cpFloat
		ratio_inv.cpFloat
		
		iSum.cpFloat
		
		bias.cpFloat
		jAcc.cpFloat
	EndStructure
	
	Structure cpSlideJoint
		constraint.cpConstraint
		anchorA.cpVect : anchorB.cpVect
		min.cpFloat : max.cpFloat
		
		r1.cpVect : r2.cpVect
		n.cpVect
		nMass.cpFloat
		
		jnAcc.cpFloat
		bias.cpFloat
	EndStructure
	
	Structure cpSimpleMotor
		constraint.cpConstraint
		rate.cpFloat
		
		iSum.cpFloat
		
		jAcc.cpFloat
	EndStructure

	; ========================================================================================================
	; Spaces: Spaces are containers for simulating objects in Chipmunk.
	; You add bodies, shapes and joints to a space and then update the space as a whole.
	; They control how all the rigid bodies, shapes, and constraints interact together.
	; ========================================================================================================

	Structure cpSpace
		iterations.l		        ; Iterations allow you to control the accuracy of the solver. Defaults to 10.
		
		gravity.cpVect 				; Global gravity applied to the space. Defaults to cpvzero. Can be overridden on a per body basis by writing custom integration functions. Changing the gravity will activate all sleeping bodies in the space.
		damping.cpFloat  			; Amount of simple damping To apply To the space. A value of 0.9 means that each body will lose 10% of its velocity per second. Defaults To 1. Like gravity, it can be overridden on a per body basis.
		
		idleSpeedThreshold.cpFloat 	; Speed threshold for a body to be considered idle. The default value of 0 means the space estimates a good threshold based on gravity.
		sleepTimeThreshold.cpFloat	; Time a group of bodies must remain idle in order to fall asleep. The default value of INFINITY disables the sleeping feature.
		
		collisionSlop.cpFloat		; Amount of overlap between shapes that is allowed. To improve stability, set this as high as you can without noticable overlapping. It defaults to 0.1.
		collisionBias.cpFloat		; Chipmunk allows fast moving objects to overlap, then fixes the overlap over time. Overlapping objects are unavoidable even if swept collisions are supported, and this is an efficient and stable way to deal with overlapping objects. The bias value controls what percentage of overlap remains unfixed after a second and defaults to ~0.2%. Valid values are in the range from 0 to 1, but using 0 is not recommended for stability reasons. The default value is calculated as cpfpow(1.0f - 0.1f, 60.0f) meaning that Chipmunk attempts to correct 10% of error ever 1/60th of a second. Note: Very very few games will need to change this value
		collisionPersistence.cpTimestamp ; The number of frames the space keeps collision solutions around for. Helps prevent jittering contacts from getting worse. This defaults to 3 and very very very few games will need to change this value.
		
		*userData ;cpDataPointer
		
		stamp.cpTimestamp
		curr_dt.cpFloat
		
		*dynamicBodies.cpArray
		*staticBodies.cpArray
		*rousedBodies.cpArray
		*sleepingComponents.cpArray
		
		shapeIDCounter.cpHashValue
		*staticShapes.cpSpatialIndex
		*dynamicShapes.cpSpatialIndex
		
		*constraints.cpArray
		
		*arbiters.cpArray
		*contactBuffersHead.cpContactBufferHeader
		*cachedArbiters.cpHashSet
		*pooledArbiters.cpArray
		
		*allocatedBuffers.cpArray
		locked.l ; unsigned int. Returns true when you cannot add/remove objects from the space. In particular, spaces are locked when in a collision callback. Instead, run your code in a post-step callback instead
		
		usesWildcards.cpBool
		*collisionHandlers.cpHashSet
		defaultHandler.cpCollisionHandler
		
		skipPostStep.cpBool
		*postStepCallbacks.cpArray
		
		*staticBody.cpBody
		_staticBody.cpBody
	EndStructure
	
	Structure cpSpaceDebugColor ; Color type To use With the space Debug drawing API.
		r.float : g.float : b.float : a.float
	EndStructure
		
	Structure cpSpaceHash
		spatialIndex.cpSpatialIndex
		
		numcells.l
		celldim.cpFloat
		
		*table.cpSpaceHashBin
		*handleSet.cpHashSet
		
		*pooledBins.cpSpaceHashBin
		*pooledHandles.cpArray
		*allocatedBuffers.cpArray
		
		stamp.cpTimestamp
	EndStructure

	; ========================================================================================================
	; [unclassified yet]
	; ========================================================================================================

	Structure cpArbiterThread
		*next.cpArbiter
		*prev.cpArbiter
	EndStructure
	
	Structure cpArbiter
		e.cpFloat
		u.cpFloat
		surface_vr.cpVect
		
		*Data ; .cpDataPointer
		
		*a.cpShape
		*b.cpShape
		
		*body_a.cpBody 
		*body_b.cpBody
		thread_a.cpArbiterThread 
		thread_b.cpArbiterThread
		
		count.l
		*contacts.cpContact
		n.cpVect 
		
	; 	// Regular, wildcard A And wildcard B collision handlers.
		*handler.cpCollisionHandler 
		*handlerA.cpCollisionHandler 
		*handlerB.cpCollisionHandler
		swapped.cpBool
		
		stamp.cpTimestamp
		state.cpArbiterState
	EndStructure
	
	
	Structure cpContactPoints
		; The position of the contact on the surface of each shape.
		pointA.cpVect
		pointB.cpVect
		; Penetration distance of the two shapes. Overlapping means it will be negative.
		; This value is calculated As cpvdot(cpvsub(point2, point1), normal) And is ignored by cpArbiterSetContactPointSet().
		distance.cpFloat
	EndStructure
	
	Structure cpContactPointSet ; A struct that wraps up the important collision Data For an arbiter.
		; The number of contact points in the set.
		count.l
		
		; The normal of the collision.
		normal.cpVect
		
		Points.cpContactPoints [#CP_MAX_CONTACTS_PER_ARBITER]
	EndStructure
	
	Structure cpContact
		r1.cpVect
		r2.cpVect
		
		nMass.cpFloat
		tMass.cpFloat
		bounce.cpFloat
		
		jnAcc.cpFloat
		jtAcc.cpFloat
		jBias.cpFloat
		
		bias.cpFloat
		
		hash.cpHashValue
	EndStructure
	
	
	Structure cpDampedSpring
		constraint.cpConstraint
		anchorA.cpVect : anchorB.cpVect
		restLength.cpFloat
		stiffness.cpFloat
		damping.cpFloat
		*cpDampedSpringForceFunc ; springForceFunc
		
		target_vrn.cpFloat
		v_coef.cpFloat
		
		r1.cpVect : r2.cpVect
		nMass.cpFloat
		n.cpVect
		
		jAcc.cpFloat
	EndStructure
	
	Structure cpDampedRotarySpring
		constraint.cpConstraint
		restAngle.cpFloat
		stiffness.cpFloat
		damping.cpFloat
		*cpDampedRotarySpringTorqueFunc ;springTorqueFunc
		
		target_wrn.cpFloat
		w_coef.cpFloat
		
		iSum.cpFloat
		jAcc.cpFloat
	EndStructure
	
	
	Structure cpPointQueryInfo ; Point query info struct.
		; The nearest shape, NULL If no shape was within range.
		*shape.cpShape
		; The closest point on the shape's surface. (in world space coordinates)
		point.cpVect
		; The distance To the point. The distance is negative If the point is inside the shape.
		distance.cpFloat
		; The gradient of the signed distance function.
		; The value should be similar To info.p/info.d, but accurate even For very small values of info.d.
		gradient.cpVect
	EndStructure
	
	Structure cpSegmentQueryInfo ; Segment query info struct.
		; The shape that was hit, Or NULL If no collision occured.
		*shape.cpShape
		; The point of impact.
		point.cpVect
		; The normal of the surface hit.
		normal.cpVect
		; The normalized distance along the query segment in the range [0, 1].
		alpha.cpFloat
	EndStructure


; ========================================================================================================
;	Import list [auto-generated]
; ========================================================================================================
; Some functions here have "_pb" extension. They are pointed to modified version of original functions inside engine,
; cause those originals returning structures by value which PB doesn't support (and which generally is a bad practice).
; 	Those functions should be called like this:
; 		Define RetStructure
; 		someFunction_pb (RetStructure) ; the RetStructure here will receive function result
; 	.. instead of:
;		Define RetStructure = someFunction ()
;
;	The original functions are excluded from import list here, but they are still present in .lib file,
;		because engine uses them internally. It is also possible to call them from PB using ASM tricks.
;	----------------------------------------------------------------------------------------------------------------------------
; 	UPD: there is also another case when this extension added: if function has "_pb" ext, but there is no *lp_out parameter,
;		then it's return was not modified but some arguments were structures byval [which PB doesn't supports too] 
;		and they were changed to pointers, functions in such cases is renamed just to know about it's modification
; ========================================================================================================

; TODO: fix cpMessage function inside ChipMunk code. It displays ASSERTS and it is very important
; TODO: make also builds with SSE/SSE2 support? not sure it will give much better perfomance (need more info about how it works), also using SSE2 is not wise cause lot of older AMD processors have no support of it and unable to execute such files
CompilerIf #PB_Compiler_Processor = #PB_Processor_x86
ImportC ".\lib\chipmunk\chipmunk.7.0.1_x86.lib"
	cpArbiterApplyCachedImpulse   (*arb.cpArbiter, dt_coef.cpFloat) As "?cpArbiterApplyCachedImpulse@@YAXPAUcpArbiter@@N@Z"
	cpArbiterApplyImpulse   (*arb.cpArbiter) As "?cpArbiterApplyImpulse@@YAXPAUcpArbiter@@@Z"
	cpArbiterCallWildcardBeginA.cpBool  (*arb.cpArbiter, *space.cpSpace) As "_cpArbiterCallWildcardBeginA"
	cpArbiterCallWildcardBeginB.cpBool  (*arb.cpArbiter, *space.cpSpace) As "_cpArbiterCallWildcardBeginB"
	cpArbiterCallWildcardPostSolveA   (*arb.cpArbiter, *space.cpSpace) As "_cpArbiterCallWildcardPostSolveA"
	cpArbiterCallWildcardPostSolveB   (*arb.cpArbiter, *space.cpSpace) As "_cpArbiterCallWildcardPostSolveB"
	cpArbiterCallWildcardPreSolveA.cpBool   (*arb.cpArbiter, *space.cpSpace) As "_cpArbiterCallWildcardPreSolveA"
	cpArbiterCallWildcardPreSolveB.cpBool   (*arb.cpArbiter, *space.cpSpace) As "_cpArbiterCallWildcardPreSolveB"
	cpArbiterCallWildcardSeparateA  (*arb.cpArbiter, *space.cpSpace) As "_cpArbiterCallWildcardSeparateA"
	cpArbiterCallWildcardSeparateB  (*arb.cpArbiter, *space.cpSpace) As "_cpArbiterCallWildcardSeparateB"
	cpArbiterGetBodies  (*arb.cpArbiter, *a.cpBody, *b.cpBody) As "_cpArbiterGetBodies"
	cpArbiterGetContactPointSet_pb  (*arb.cpArbiter, *lp_out.cpContactPointSet) As "?cpArbiterGetContactPointSet_pb@@YAXPBUcpArbiter@@PAUcpContactPointSet@@@Z"
	cpArbiterGetCount.l   (*arb.cpArbiter) As "_cpArbiterGetCount"
	cpArbiterGetDepth.cpFloat   (*arb.cpArbiter, i.l) As "_cpArbiterGetDepth"
	cpArbiterGetFriction.cpFloat  (*arb.cpArbiter) As "_cpArbiterGetFriction"
	cpArbiterGetNormal_pb   (*arb.cpArbiter, *lp_out.cpVect) As "?cpArbiterGetNormal_pb@@YAXPBUcpArbiter@@PAUcpVect@@@Z"
	cpArbiterGetPointA_pb   (*arb.cpArbiter, i.l, *lp_out.cpVect) As "?cpArbiterGetPointA_pb@@YAXPBUcpArbiter@@HPAUcpVect@@@Z"
	cpArbiterGetPointB_pb   (*arb.cpArbiter, i.l, *lp_out.cpVect) As "?cpArbiterGetPointB_pb@@YAXPBUcpArbiter@@HPAUcpVect@@@Z"
	cpArbiterGetRestitution.cpFloat   (*arb.cpArbiter) As "_cpArbiterGetRestitution"
	cpArbiterGetShapes  (*arb.cpArbiter, *a.cpShape, *b.cpShape) As "_cpArbiterGetShapes"
	cpArbiterGetSurfaceVelocity_pb  (*arb.cpArbiter, *lp_out.cpVect) As "?cpArbiterGetSurfaceVelocity_pb@@YAXPAUcpArbiter@@PAUcpVect@@@Z"
	cpArbiterGetUserData.l  (*arb.cpArbiter) As "_cpArbiterGetUserData";  = *cpDataPointer
	cpArbiterIgnore.cpBool  (*arb.cpArbiter) As "_cpArbiterIgnore"
	cpArbiterInit.l   (*arb.cpArbiter, *a.cpShape, *b.cpShape) As "?cpArbiterInit@@YAPAUcpArbiter@@PAU1@PAUcpShape@@1@Z";  = *cpArbiter
	cpArbiterIsFirstContact.cpBool  (*arb.cpArbiter) As "_cpArbiterIsFirstContact"
	cpArbiterIsRemoval.cpBool   (*arb.cpArbiter) As "_cpArbiterIsRemoval"
	cpArbiterPreStep  (*arb.cpArbiter, dt.cpFloat, slop.cpFloat, bias.cpFloat) As "?cpArbiterPreStep@@YAXPAUcpArbiter@@NNN@Z"
	cpArbiterSetContactPointSet   (*arb.cpArbiter, *set.cpContactPointSet) As "_cpArbiterSetContactPointSet"
	cpArbiterSetFriction  (*arb.cpArbiter, friction.cpFloat) As "_cpArbiterSetFriction"
	cpArbiterSetRestitution   (*arb.cpArbiter, restitution.cpFloat) As "_cpArbiterSetRestitution"
	cpArbiterSetSurfaceVelocity_pb  (*arb.cpArbiter, *vr.cpVect) As "?cpArbiterSetSurfaceVelocity_pb@@YAXPAUcpArbiter@@PAUcpVect@@@Z"
	cpArbiterSetUserData  (*arb.cpArbiter, *cpDataPointer) As "_cpArbiterSetUserData"
	cpArbiterTotalImpulse_pb  (*arb.cpArbiter, *lp_out.cpVect) As "?cpArbiterTotalImpulse_pb@@YAXPBUcpArbiter@@PAUcpVect@@@Z"
	cpArbiterTotalKE.cpFloat  (*arb.cpArbiter) As "_cpArbiterTotalKE"
	cpArbiterUnthread   (*arb.cpArbiter) As "?cpArbiterUnthread@@YAXPAUcpArbiter@@@Z"
	cpArbiterUpdate   (*arb.cpArbiter, *info.cpCollisionInfo, *space.cpSpace) As "?cpArbiterUpdate@@YAXPAUcpArbiter@@PAUcpCollisionInfo@@PAUcpSpace@@@Z"

	cpAreaForCircle.cpFloat   (r1.cpFloat, r2.cpFloat) As "_cpAreaForCircle"
	cpAreaForPoly.cpFloat   (count.l, *verts.cpVect, r.cpFloat) As "_cpAreaForPoly"
	cpAreaForSegment_pb.cpFloat   (*a.cpVect, *b.cpVect, r.cpFloat) As "?cpAreaForSegment_pb@@YANPAUcpVect@@0N@Z"

	cpArrayContains.cpBool  (*arr.cpArray, *ptr) As "?cpArrayContains@@YAHPAUcpArray@@PAX@Z"
	cpArrayDeleteObj  (*arr.cpArray, *obj) As "?cpArrayDeleteObj@@YAXPAUcpArray@@PAX@Z"
	cpArrayFree   (*arr.cpArray) As "?cpArrayFree@@YAXPAUcpArray@@@Z"
	cpArrayFreeEach   (*arr.cpArray, *freeFunc) As "?cpArrayFreeEach@@YAXPAUcpArray@@P6AXPAX@Z@Z"
	cpArrayNew.l  (size.l) As "?cpArrayNew@@YAPAUcpArray@@H@Z";  = *cpArray
	cpArrayPop.l  (*arr.cpArray) As "?cpArrayPop@@YAPAXPAUcpArray@@@Z";  = *void
	cpArrayPush   (*arr.cpArray, *object) As "?cpArrayPush@@YAXPAUcpArray@@PAX@Z"

	cpBBTreeAlloc.l   () As "_cpBBTreeAlloc";  = *cpBBTree
	cpBBTreeInit.l  (*tree.cpBBTree, *cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "_cpBBTreeInit";  = *cpSpatialIndex
	cpBBTreeNew.l   (*cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "_cpBBTreeNew";  = *cpSpatialIndex
	cpBBTreeOptimize  (*index.cpSpatialIndex) As "_cpBBTreeOptimize"
	cpBBTreeSetVelocityFunc   (*index.cpSpatialIndex, *cpBBTreeVelocityFunc) As "_cpBBTreeSetVelocityFunc"

	cpBodyAccumulateMassFromShapes  (*body.cpBody) As "?cpBodyAccumulateMassFromShapes@@YAXPAUcpBody@@@Z"
	cpBodyActivate  (*body.cpBody) As "_cpBodyActivate"
	cpBodyActivateStatic  (*body.cpBody, *filter.cpShape) As "_cpBodyActivateStatic"
	cpBodyAddShape  (*body.cpBody, *shape.cpShape) As "?cpBodyAddShape@@YAXPAUcpBody@@PAUcpShape@@@Z"
	cpBodyAlloc.l   () As "_cpBodyAlloc";  = *cpBody
	cpBodyApplyForceAtLocalPoint_pb   (*body.cpBody, *force.cpVect, *point.cpVect) As "?cpBodyApplyForceAtLocalPoint_pb@@YAXPAUcpBody@@PAUcpVect@@1@Z"
	cpBodyApplyForceAtWorldPoint_pb   (*body.cpBody, *force.cpVect, *point.cpVect) As "?cpBodyApplyForceAtWorldPoint_pb@@YAXPAUcpBody@@PAUcpVect@@1@Z"
	cpBodyApplyImpulseAtLocalPoint_pb   (*body.cpBody, *impulse.cpVect, *point.cpVect) As "?cpBodyApplyImpulseAtLocalPoint_pb@@YAXPAUcpBody@@PAUcpVect@@1@Z"
	cpBodyApplyImpulseAtWorldPoint_pb   (*body.cpBody, *impulse.cpVect, *point.cpVect) As "?cpBodyApplyImpulseAtWorldPoint_pb@@YAXPAUcpBody@@PAUcpVect@@1@Z"
	cpBodyDestroy   (*body.cpBody) As "_cpBodyDestroy"
	cpBodyEachArbiter   (*body.cpBody, *cpBodyArbiterIteratorFunc, *data) As "_cpBodyEachArbiter"
	cpBodyEachConstraint  (*body.cpBody, *cpBodyConstraintIteratorFunc, *data) As "_cpBodyEachConstraint"
	cpBodyEachShape   (*body.cpBody, *cpBodyShapeIteratorFunc, *data) As "_cpBodyEachShape"
	cpBodyFree  (*body.cpBody) As "_cpBodyFree"
	cpBodyGetAngle.cpFloat  (*body.cpBody) As "_cpBodyGetAngle"
	cpBodyGetAngularVelocity.cpFloat  (*body.cpBody) As "_cpBodyGetAngularVelocity"
	cpBodyGetCenterOfGravity_pb   (*body.cpBody, *lp_out.cpVect) As "?cpBodyGetCenterOfGravity_pb@@YAXPBUcpBody@@PAUcpVect@@@Z"
	cpBodyGetForce_pb   (*body.cpBody, *lp_out.cpVect) As "?cpBodyGetForce_pb@@YAXPBUcpBody@@PAUcpVect@@@Z"
	cpBodyGetMass.cpFloat   (*body.cpBody) As "_cpBodyGetMass"
	cpBodyGetMoment.cpFloat   (*body.cpBody) As "_cpBodyGetMoment"
	cpBodyGetPosition_pb  (*body.cpBody, *lp_out.cpVect) As "?cpBodyGetPosition_pb@@YAXPBUcpBody@@PAUcpVect@@@Z"
	cpBodyGetRotation_pb  (*body.cpBody, *lp_out.cpVect) As "?cpBodyGetRotation_pb@@YAXPBUcpBody@@PAUcpVect@@@Z"
	cpBodyGetSpace.l  (*body.cpBody) As "_cpBodyGetSpace";  = *cpSpace
	cpBodyGetTorque.cpFloat   (*body.cpBody) As "_cpBodyGetTorque"
	cpBodyGetType.cpBodyType  (*body.cpBody) As "_cpBodyGetType"
	cpBodyGetUserData.l   (*body.cpBody) As "_cpBodyGetUserData";  = *cpDataPointer
	cpBodyGetVelocity_pb  (*body.cpBody, *lp_out.cpVect) As "?cpBodyGetVelocity_pb@@YAXPBUcpBody@@PAUcpVect@@@Z"
	cpBodyGetVelocityAtLocalPoint_pb  (*body.cpBody, *point.cpVect, *lp_out.cpVect) As "?cpBodyGetVelocityAtLocalPoint_pb@@YAXPBUcpBody@@PAUcpVect@@1@Z"
	cpBodyGetVelocityAtWorldPoint_pb  (*body.cpBody, *point.cpVect, *lp_out.cpVect) As "?cpBodyGetVelocityAtWorldPoint_pb@@YAXPBUcpBody@@PAUcpVect@@1@Z"
	cpBodyInit.l  (*body.cpBody, mass.cpFloat, moment.cpFloat) As "_cpBodyInit";  = *cpBody
	cpBodyIsSleeping.cpBool   (*body.cpBody) As "_cpBodyIsSleeping"
	cpBodyKineticEnergy.cpFloat   (*body.cpBody) As "_cpBodyKineticEnergy"
	cpBodyLocalToWorld_pb   (*body.cpBody, *point.cpVect, *lp_out.cpVect) As "?cpBodyLocalToWorld_pb@@YAXPBUcpBody@@PBUcpVect@@PAU2@@Z"
	cpBodyNew.l   (mass.cpFloat, moment.cpFloat) As "_cpBodyNew";  = *cpBody
	cpBodyNewKinematic.l  () As "_cpBodyNewKinematic";  = *cpBody
	cpBodyNewStatic.l   () As "_cpBodyNewStatic";  = *cpBody
	cpBodyRemoveConstraint  (*body.cpBody, *constraint.cpConstraint) As "?cpBodyRemoveConstraint@@YAXPAUcpBody@@PAUcpConstraint@@@Z"
	cpBodyRemoveShape   (*body.cpBody, *shape.cpShape) As "?cpBodyRemoveShape@@YAXPAUcpBody@@PAUcpShape@@@Z"
	cpBodySetAngle  (*body.cpBody, angle.cpFloat) As "_cpBodySetAngle"
	cpBodySetAngularVelocity  (*body.cpBody, angularVelocity.cpFloat) As "_cpBodySetAngularVelocity"
	cpBodySetCenterOfGravity_pb   (*body.cpBody, *cog.cpVect) As "?cpBodySetCenterOfGravity_pb@@YAXPAUcpBody@@PAUcpVect@@@Z"
	cpBodySetForce_pb   (*body.cpBody, *force.cpVect) As "?cpBodySetForce_pb@@YAXPAUcpBody@@PAUcpVect@@@Z"
	cpBodySetMass   (*body.cpBody, mass.cpFloat) As "_cpBodySetMass"
	cpBodySetMoment   (*body.cpBody, moment.cpFloat) As "_cpBodySetMoment"
	cpBodySetPosition_pb  (*body.cpBody, *position.cpVect) As "?cpBodySetPosition_pb@@YAXPAUcpBody@@PAUcpVect@@@Z"
	cpBodySetPositionUpdateFunc   (*body.cpBody, *cpBodyPositionFunc) As "_cpBodySetPositionUpdateFunc"
	cpBodySetTorque   (*body.cpBody, torque.cpFloat) As "_cpBodySetTorque"
	cpBodySetType   (*body.cpBody, type.cpBodyType) As "_cpBodySetType"
	cpBodySetUserData   (*body.cpBody, *cpDataPointer) As "_cpBodySetUserData"
	cpBodySetVelocity_pb  (*body.cpBody, *velocity.cpVect) As "?cpBodySetVelocity_pb@@YAXPAUcpBody@@PAUcpVect@@@Z"
	cpBodySetVelocityUpdateFunc   (*body.cpBody, *cpBodyVelocityFunc) As "_cpBodySetVelocityUpdateFunc"
	cpBodySleep   (*body.cpBody) As "_cpBodySleep"
	cpBodySleepWithGroup  (*body.cpBody, *group.cpBody) As "_cpBodySleepWithGroup"
	cpBodyUpdatePosition  (*body.cpBody, dt.cpFloat) As "_cpBodyUpdatePosition"
	cpBodyUpdateVelocity_pb   (*body.cpBody, *gravity.cpVect, damping.cpFloat, dt.cpFloat) As "?cpBodyUpdateVelocity_pb@@YAXPAUcpBody@@PAUcpVect@@NN@Z"
	cpBodyWorldToLocal_pb   (*body.cpBody, *point.cpVect, *lp_out.cpVect) As "?cpBodyWorldToLocal_pb@@YAXPBUcpBody@@PBUcpVect@@PAU2@@Z"

	cpBoxShapeInit.l  (*poly.cpPolyShape, *body.cpBody, width.cpFloat, height.cpFloat, radius.cpFloat) As "_cpBoxShapeInit";  = *cpPolyShape
	cpBoxShapeInit2_pb.l  (*poly.cpPolyShape, *body.cpBody, *box.cpBB, radius.cpFloat) As "?cpBoxShapeInit2_pb@@YAPAUcpPolyShape@@PAU1@PAUcpBody@@PAUcpBB@@N@Z";  = *cpPolyShape
	cpBoxShapeNew.l   (*body.cpBody, width.cpFloat, height.cpFloat, radius.cpFloat) As "_cpBoxShapeNew";  = *cpShape
	cpBoxShapeNew2_pb.l   (*body.cpBody, *box.cpBB, radius.cpFloat) As "?cpBoxShapeNew2_pb@@YAPAUcpShape@@PAUcpBody@@PAUcpBB@@N@Z";  = *cpShape

	cpCentroidForPoly_pb  (count.l, *verts.cpVect, *lp_out.cpVect) As "?cpCentroidForPoly_pb@@YAXHPBUcpVect@@PAU1@@Z"
	cpCheckSignedArea_pb.cpBool   (*a.cpVect, *b.cpVect, *c.cpVect) As "?cpCheckSignedArea_pb@@YAHPBUcpVect@@00@Z"

	cpCircleShapeAlloc.l  () As "_cpCircleShapeAlloc";  = *cpCircleShape
	cpCircleShapeGetOffset_pb   (*shape.cpShape, *lp_out.cpVect) As "?cpCircleShapeGetOffset_pb@@YAXPBUcpShape@@PAUcpVect@@@Z"
	cpCircleShapeGetRadius.cpFloat  (*shape.cpShape) As "_cpCircleShapeGetRadius"
	cpCircleShapeInit_pb.l  (*circle.cpCircleShape, *body.cpBody, radius.cpFloat, *offset.cpVect) As "?cpCircleShapeInit_pb@@YAPAUcpCircleShape@@PAU1@PAUcpBody@@NPAUcpVect@@@Z";  = *cpCircleShape
	cpCircleShapeNew_pb.l   (*body.cpBody, radius.cpFloat, *offset.cpVect) As "?cpCircleShapeNew_pb@@YAPAUcpShape@@PAUcpBody@@NPAUcpVect@@@Z";  = *cpShape
	cpCircleShapeSetOffset_pb   (*shape.cpShape, *offset.cpVect) As "?cpCircleShapeSetOffset_pb@@YAXPAUcpShape@@PAUcpVect@@@Z"
	cpCircleShapeSetRadius  (*shape.cpShape, radius.cpFloat) As "_cpCircleShapeSetRadius"

	cpCollide_pb  (*a.cpShape, *b.cpShape, id.cpCollisionID, *contacts.cpContact, *lp_out.cpCollisionInfo) As "?cpCollide_pb@@YAXPBUcpShape@@0IPAUcpContact@@PAUcpCollisionInfo@@@Z"

	cpConstraintDestroy   (*constraint.cpConstraint) As "_cpConstraintDestroy"
	cpConstraintFree  (*constraint.cpConstraint) As "_cpConstraintFree"
	cpConstraintGetBodyA.l  (*constraint.cpConstraint) As "_cpConstraintGetBodyA";  = *cpBody
	cpConstraintGetBodyB.l  (*constraint.cpConstraint) As "_cpConstraintGetBodyB";  = *cpBody
	cpConstraintGetCollideBodies.cpBool   (*constraint.cpConstraint) As "_cpConstraintGetCollideBodies"
	cpConstraintGetErrorBias.cpFloat  (*constraint.cpConstraint) As "_cpConstraintGetErrorBias"
	cpConstraintGetImpulse.cpFloat  (*constraint.cpConstraint) As "_cpConstraintGetImpulse"
	cpConstraintGetMaxBias.cpFloat  (*constraint.cpConstraint) As "_cpConstraintGetMaxBias"
	cpConstraintGetMaxForce.cpFloat   (*constraint.cpConstraint) As "_cpConstraintGetMaxForce"
	cpConstraintGetPostSolveFunc.l  (*constraint.cpConstraint) As "_cpConstraintGetPostSolveFunc";  = cpConstraintPostSolveFunc
	cpConstraintGetPreSolveFunc.l   (*constraint.cpConstraint) As "_cpConstraintGetPreSolveFunc";  = cpConstraintPreSolveFunc
	cpConstraintGetSpace.l  (*constraint.cpConstraint) As "_cpConstraintGetSpace";  = *cpSpace
	cpConstraintGetUserData.l   (*constraint.cpConstraint) As "_cpConstraintGetUserData";  = *cpDataPointer
	cpConstraintInit  (*constraint.cpConstraint, *klass.cpConstraintClass, *a.cpBody, *b.cpBody) As "?cpConstraintInit@@YAXPAUcpConstraint@@PBUcpConstraintClass@@PAUcpBody@@2@Z"
	cpConstraintIsDampedRotarySpring.cpBool   (*constraint.cpConstraint) As "_cpConstraintIsDampedRotarySpring"
	cpConstraintIsDampedSpring.cpBool   (*constraint.cpConstraint) As "_cpConstraintIsDampedSpring"
	cpConstraintIsGearJoint.cpBool  (*constraint.cpConstraint) As "_cpConstraintIsGearJoint"
	cpConstraintIsGrooveJoint.cpBool  (*constraint.cpConstraint) As "_cpConstraintIsGrooveJoint"
	cpConstraintIsPinJoint.cpBool   (*constraint.cpConstraint) As "_cpConstraintIsPinJoint"
	cpConstraintIsPivotJoint.cpBool   (*constraint.cpConstraint) As "_cpConstraintIsPivotJoint"
	cpConstraintIsRatchetJoint.cpBool   (*constraint.cpConstraint) As "_cpConstraintIsRatchetJoint"
	cpConstraintIsRotaryLimitJoint.cpBool   (*constraint.cpConstraint) As "_cpConstraintIsRotaryLimitJoint"
	cpConstraintIsSimpleMotor.cpBool  (*constraint.cpConstraint) As "_cpConstraintIsSimpleMotor"
	cpConstraintIsSlideJoint.cpBool   (*constraint.cpConstraint) As "_cpConstraintIsSlideJoint"
	cpConstraintSetCollideBodies  (*constraint.cpConstraint, collideBodies.cpBool) As "_cpConstraintSetCollideBodies"
	cpConstraintSetErrorBias  (*constraint.cpConstraint, errorBias.cpFloat) As "_cpConstraintSetErrorBias"
	cpConstraintSetMaxBias  (*constraint.cpConstraint, maxBias.cpFloat) As "_cpConstraintSetMaxBias"
	cpConstraintSetMaxForce   (*constraint.cpConstraint, maxForce.cpFloat) As "_cpConstraintSetMaxForce"
	cpConstraintSetPostSolveFunc  (*constraint.cpConstraint, *cpConstraintPostSolveFunc) As "_cpConstraintSetPostSolveFunc"
	cpConstraintSetPreSolveFunc   (*constraint.cpConstraint, *cpConstraintPreSolveFunc) As "_cpConstraintSetPreSolveFunc"
	cpConstraintSetUserData   (*constraint.cpConstraint, *cpDataPointer) As "_cpConstraintSetUserData"

	cpContactBufferGetArray.l   (*space.cpSpace) As "?cpContactBufferGetArray@@YAPAUcpContact@@PAUcpSpace@@@Z";  = *cpContact
	cpConvexHull.l  (count.l, *verts.cpVect, *result.cpVect, *first.Long, tol.cpFloat) As "_cpConvexHull"

	cpDampedRotarySpringAlloc.l   () As "_cpDampedRotarySpringAlloc";  = *cpDampedRotarySpring
	cpDampedRotarySpringGetDamping.cpFloat  (*constraint.cpConstraint) As "_cpDampedRotarySpringGetDamping"
	cpDampedRotarySpringGetRestAngle.cpFloat  (*constraint.cpConstraint) As "_cpDampedRotarySpringGetRestAngle"
	cpDampedRotarySpringGetSpringTorqueFunc.l   (*constraint.cpConstraint) As "_cpDampedRotarySpringGetSpringTorqueFunc";  = cpDampedRotarySpringTorqueFunc
	cpDampedRotarySpringGetStiffness.cpFloat  (*constraint.cpConstraint) As "_cpDampedRotarySpringGetStiffness"
	cpDampedRotarySpringInit.l  (*spring.cpDampedRotarySpring, *a.cpBody, *b.cpBody, restAngle.cpFloat, stiffness.cpFloat, damping.cpFloat) As "_cpDampedRotarySpringInit";  = *cpDampedRotarySpring
	cpDampedRotarySpringNew.l   (*a.cpBody, *b.cpBody, restAngle.cpFloat, stiffness.cpFloat, damping.cpFloat) As "_cpDampedRotarySpringNew";  = *cpConstraint
	cpDampedRotarySpringSetDamping  (*constraint.cpConstraint, damping.cpFloat) As "_cpDampedRotarySpringSetDamping"
	cpDampedRotarySpringSetRestAngle  (*constraint.cpConstraint, restAngle.cpFloat) As "_cpDampedRotarySpringSetRestAngle"
	cpDampedRotarySpringSetSpringTorqueFunc   (*constraint.cpConstraint, *cpDampedRotarySpringTorqueFunc) As "_cpDampedRotarySpringSetSpringTorqueFunc"
	cpDampedRotarySpringSetStiffness  (*constraint.cpConstraint, stiffness.cpFloat) As "_cpDampedRotarySpringSetStiffness"

	cpDampedSpringAlloc.l   () As "_cpDampedSpringAlloc";  = *cpDampedSpring
	cpDampedSpringGetAnchorA_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpDampedSpringGetAnchorA_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpDampedSpringGetAnchorB_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpDampedSpringGetAnchorB_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpDampedSpringGetDamping.cpFloat  (*constraint.cpConstraint) As "_cpDampedSpringGetDamping"
	cpDampedSpringGetRestLength.cpFloat   (*constraint.cpConstraint) As "_cpDampedSpringGetRestLength"
	cpDampedSpringGetSpringForceFunc.l  (*constraint.cpConstraint) As "_cpDampedSpringGetSpringForceFunc";  = cpDampedSpringForceFunc
	cpDampedSpringGetStiffness.cpFloat  (*constraint.cpConstraint) As "_cpDampedSpringGetStiffness"
	cpDampedSpringInit_pb.l   (*spring.cpDampedSpring, *a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect, restLength.cpFloat, stiffness.cpFloat, damping.cpFloat) As "?cpDampedSpringInit_pb@@YAPAUcpDampedSpring@@PAU1@PAUcpBody@@1PAUcpVect@@2NNN@Z";  = *cpDampedSpring
	cpDampedSpringNew_pb.l  (*a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect, restLength.cpFloat, stiffness.cpFloat, damping.cpFloat) As "?cpDampedSpringNew_pb@@YAPAUcpConstraint@@PAUcpBody@@0PAUcpVect@@1NNN@Z";  = *cpConstraint
	cpDampedSpringSetAnchorA_pb   (*constraint.cpConstraint, *anchorA.cpVect) As "?cpDampedSpringSetAnchorA_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpDampedSpringSetAnchorB_pb   (*constraint.cpConstraint, *anchorB.cpVect) As "?cpDampedSpringSetAnchorB_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpDampedSpringSetDamping  (*constraint.cpConstraint, damping.cpFloat) As "_cpDampedSpringSetDamping"
	cpDampedSpringSetRestLength   (*constraint.cpConstraint, restLength.cpFloat) As "_cpDampedSpringSetRestLength"
	cpDampedSpringSetSpringForceFunc  (*constraint.cpConstraint, *cpDampedSpringForceFunc) As "_cpDampedSpringSetSpringForceFunc"
	cpDampedSpringSetStiffness  (*constraint.cpConstraint, stiffness.cpFloat) As "_cpDampedSpringSetStiffness"

	cpGearJointAlloc.l  () As "_cpGearJointAlloc";  = *cpGearJoint
	cpGearJointGetPhase.cpFloat   (*constraint.cpConstraint) As "_cpGearJointGetPhase"
	cpGearJointGetRatio.cpFloat   (*constraint.cpConstraint) As "_cpGearJointGetRatio"
	cpGearJointInit.l   (*joint.cpGearJoint, *a.cpBody, *b.cpBody, phase.cpFloat, ratio.cpFloat) As "_cpGearJointInit";  = *cpGearJoint
	cpGearJointNew.l  (*a.cpBody, *b.cpBody, phase.cpFloat, ratio.cpFloat) As "_cpGearJointNew";  = *cpConstraint
	cpGearJointSetPhase   (*constraint.cpConstraint, phase.cpFloat) As "_cpGearJointSetPhase"
	cpGearJointSetRatio   (*constraint.cpConstraint, ratio.cpFloat) As "_cpGearJointSetRatio"

	cpGrooveJointAlloc.l  () As "_cpGrooveJointAlloc";  = *cpGrooveJoint
	cpGrooveJointGetAnchorB_pb  (*constraint.cpConstraint, *lp_out.cpVect) As "?cpGrooveJointGetAnchorB_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpGrooveJointGetGrooveA_pb  (*constraint.cpConstraint, *lp_out.cpVect) As "?cpGrooveJointGetGrooveA_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpGrooveJointGetGrooveB_pb  (*constraint.cpConstraint, *lp_out.cpVect) As "?cpGrooveJointGetGrooveB_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpGrooveJointInit_pb.l  (*joint.cpGrooveJoint, *a.cpBody, *b.cpBody, *groove_a.cpVect, *groove_b.cpVect, *anchorB.cpVect) As "?cpGrooveJointInit_pb@@YAPAUcpGrooveJoint@@PAU1@PAUcpBody@@1PAUcpVect@@22@Z";  = *cpGrooveJoint
	cpGrooveJointNew_pb.l   (*a.cpBody, *b.cpBody, *groove_a.cpVect, *groove_b.cpVect, *anchorB.cpVect) As "?cpGrooveJointNew_pb@@YAPAUcpConstraint@@PAUcpBody@@0PAUcpVect@@11@Z";  = *cpConstraint
	cpGrooveJointSetAnchorB_pb  (*constraint.cpConstraint, *anchorB.cpVect) As "?cpGrooveJointSetAnchorB_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpGrooveJointSetGrooveA_pb  (*constraint.cpConstraint, *value.cpVect) As "?cpGrooveJointSetGrooveA_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpGrooveJointSetGrooveB_pb  (*constraint.cpConstraint, *value.cpVect) As "?cpGrooveJointSetGrooveB_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"

	cpHashSetCount.l  (*set.cpHashSet) As "?cpHashSetCount@@YAHPAUcpHashSet@@@Z"
	cpHashSetEach   (*set.cpHashSet, *cpHashSetIteratorFunc, *data) As "?cpHashSetEach@@YAXPAUcpHashSet@@P6AXPAX1@Z1@Z"
	cpHashSetFilter   (*set.cpHashSet, *cpHashSetFilterFunc, *data) As "?cpHashSetFilter@@YAXPAUcpHashSet@@P6AHPAX1@Z1@Z"
	cpHashSetFind.l   (*set.cpHashSet, hash.cpHashValue, *ptr) As "?cpHashSetFind@@YAPAXPAUcpHashSet@@IPAX@Z";  = *void
	cpHashSetFree   (*set.cpHashSet) As "?cpHashSetFree@@YAXPAUcpHashSet@@@Z"
	cpHashSetInsert.l   (*set.cpHashSet, hash.cpHashValue, *ptr, *cpHashSetTransFunc, *data) As "?cpHashSetInsert@@YAPAXPAUcpHashSet@@IPAXP6APAX11@Z1@Z";  = *void
	cpHashSetNew.l  (size.l, *cpHashSetEqlFunc) As "?cpHashSetNew@@YAPAUcpHashSet@@HP6AHPAX0@Z@Z";  = *cpHashSet
	cpHashSetRemove.l   (*set.cpHashSet, hash.cpHashValue, *ptr) As "?cpHashSetRemove@@YAPAXPAUcpHashSet@@IPAX@Z";  = *void
	cpHashSetSetDefaultValue  (*set.cpHashSet, *default_value) As "?cpHashSetSetDefaultValue@@YAXPAUcpHashSet@@PAX@Z"

	cpLoopIndexes   (*verts.cpVect, count.l, *start.Long, *end.Long) As "?cpLoopIndexes@@YAXPBUcpVect@@HPAH1@Z"
; 	cpMessage   (*condition.Ascii, *file.Ascii, line.l, isError.l, isHardError.l, *message.Ascii, ...) As "_cpMessage"

	cpMomentForBox.cpFloat  (m.cpFloat, width.cpFloat, height.cpFloat) As "_cpMomentForBox"
	cpMomentForBox2_pb.cpFloat  (m.cpFloat, *box.cpBB) As "?cpMomentForBox2_pb@@YANNPAUcpBB@@@Z"
	cpMomentForCircle_pb.cpFloat  (m.cpFloat, r1.cpFloat, r2.cpFloat, *offset.cpVect) As "?cpMomentForCircle_pb@@YANNNNPAUcpVect@@@Z"
	cpMomentForPoly_pb.cpFloat  (m.cpFloat, count.l, *verts.cpVect, *offset.cpVect, r.cpFloat) As "?cpMomentForPoly_pb@@YANNHPBUcpVect@@PAU1@N@Z"
	cpMomentForSegment_pb.cpFloat   (m.cpFloat, *a.cpVect, *b.cpVect, r.cpFloat) As "?cpMomentForSegment_pb@@YANNPAUcpVect@@0N@Z"

	cpPinJointAlloc.l   () As "_cpPinJointAlloc";  = *cpPinJoint
	cpPinJointGetAnchorA_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpPinJointGetAnchorA_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpPinJointGetAnchorB_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpPinJointGetAnchorB_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpPinJointGetDist.cpFloat   (*constraint.cpConstraint) As "_cpPinJointGetDist"
	cpPinJointInit_pb.l   (*joint.cpPinJoint, *a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect) As "?cpPinJointInit_pb@@YAPAUcpPinJoint@@PAU1@PAUcpBody@@1PAUcpVect@@2@Z";  = *cpPinJoint
	cpPinJointNew_pb.l  (*a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect) As "?cpPinJointNew_pb@@YAPAUcpConstraint@@PAUcpBody@@0PAUcpVect@@1@Z";  = *cpConstraint
	cpPinJointSetAnchorA_pb   (*constraint.cpConstraint, *anchorA.cpVect) As "?cpPinJointSetAnchorA_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpPinJointSetAnchorB_pb   (*constraint.cpConstraint, *anchorB.cpVect) As "?cpPinJointSetAnchorB_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpPinJointSetDist   (*constraint.cpConstraint, dist.cpFloat) As "_cpPinJointSetDist"

	cpPivotJointAlloc.l   () As "_cpPivotJointAlloc";  = *cpPivotJoint
	cpPivotJointGetAnchorA_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpPivotJointGetAnchorA_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpPivotJointGetAnchorB_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpPivotJointGetAnchorB_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpPivotJointInit_pb.l   (*joint.cpPivotJoint, *a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect) As "?cpPivotJointInit_pb@@YAPAUcpPivotJoint@@PAU1@PAUcpBody@@1PAUcpVect@@2@Z";  = *cpPivotJoint
	cpPivotJointNew2_pb.l   (*a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect) As "?cpPivotJointNew2_pb@@YAPAUcpConstraint@@PAUcpBody@@0PAUcpVect@@1@Z";  = *cpConstraint
	cpPivotJointNew_pb.l  (*a.cpBody, *b.cpBody, *pivot.cpVect) As "?cpPivotJointNew_pb@@YAPAUcpConstraint@@PAUcpBody@@0PAUcpVect@@@Z";  = *cpConstraint
	cpPivotJointSetAnchorA_pb   (*constraint.cpConstraint, *anchorA.cpVect) As "?cpPivotJointSetAnchorA_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpPivotJointSetAnchorB_pb   (*constraint.cpConstraint, *anchorB.cpVect) As "?cpPivotJointSetAnchorB_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"

	cpPolyShapeAlloc.l  () As "_cpPolyShapeAlloc";  = *cpPolyShape
	cpPolyShapeGetCount.l   (*shape.cpShape) As "_cpPolyShapeGetCount"
	cpPolyShapeGetRadius.cpFloat  (*shape.cpShape) As "_cpPolyShapeGetRadius"
	cpPolyShapeGetVert_pb   (*shape.cpShape, i.l, *lp_out.cpVect) As "?cpPolyShapeGetVert_pb@@YAXPBUcpShape@@HPAUcpVect@@@Z"
	cpPolyShapeInit_pb.l  (*poly.cpPolyShape, *body.cpBody, count.l, *verts.cpVect, *transform.cpTransform, radius.cpFloat) As "?cpPolyShapeInit_pb@@YAPAUcpPolyShape@@PAU1@PAUcpBody@@HPBUcpVect@@PAUcpTransform@@N@Z";  = *cpPolyShape
	cpPolyShapeInitRaw.l  (*poly.cpPolyShape, *body.cpBody, count.l, *verts.cpVect, radius.cpFloat) As "_cpPolyShapeInitRaw";  = *cpPolyShape
	cpPolyShapeNew_pb.l   (*body.cpBody, count.l, *verts.cpVect, *transform.cpTransform, radius.cpFloat) As "?cpPolyShapeNew_pb@@YAPAUcpShape@@PAUcpBody@@HPBUcpVect@@PAUcpTransform@@N@Z";  = *cpShape
	cpPolyShapeNewRaw.l   (*body.cpBody, count.l, *verts.cpVect, radius.cpFloat) As "_cpPolyShapeNewRaw";  = *cpShape
	cpPolyShapeSetRadius  (*shape.cpShape, radius.cpFloat) As "_cpPolyShapeSetRadius"
	cpPolyShapeSetVerts_pb  (*shape.cpShape, count.l, *verts.cpVect, *transform.cpTransform) As "?cpPolyShapeSetVerts_pb@@YAXPAUcpShape@@HPAUcpVect@@PAUcpTransform@@@Z"
	cpPolyShapeSetVertsRaw  (*shape.cpShape, count.l, *verts.cpVect) As "_cpPolyShapeSetVertsRaw"

	cpRatchetJointAlloc.l   () As "_cpRatchetJointAlloc";  = *cpRatchetJoint
	cpRatchetJointGetAngle.cpFloat  (*constraint.cpConstraint) As "_cpRatchetJointGetAngle"
	cpRatchetJointGetPhase.cpFloat  (*constraint.cpConstraint) As "_cpRatchetJointGetPhase"
	cpRatchetJointGetRatchet.cpFloat  (*constraint.cpConstraint) As "_cpRatchetJointGetRatchet"
	cpRatchetJointInit.l  (*joint.cpRatchetJoint, *a.cpBody, *b.cpBody, phase.cpFloat, ratchet.cpFloat) As "_cpRatchetJointInit";  = *cpRatchetJoint
	cpRatchetJointNew.l   (*a.cpBody, *b.cpBody, phase.cpFloat, ratchet.cpFloat) As "_cpRatchetJointNew";  = *cpConstraint
	cpRatchetJointSetAngle  (*constraint.cpConstraint, angle.cpFloat) As "_cpRatchetJointSetAngle"
	cpRatchetJointSetPhase  (*constraint.cpConstraint, phase.cpFloat) As "_cpRatchetJointSetPhase"
	cpRatchetJointSetRatchet  (*constraint.cpConstraint, ratchet.cpFloat) As "_cpRatchetJointSetRatchet"

	cpRotaryLimitJointAlloc.l   () As "_cpRotaryLimitJointAlloc";  = *cpRotaryLimitJoint
	cpRotaryLimitJointGetMax.cpFloat  (*constraint.cpConstraint) As "_cpRotaryLimitJointGetMax"
	cpRotaryLimitJointGetMin.cpFloat  (*constraint.cpConstraint) As "_cpRotaryLimitJointGetMin"
	cpRotaryLimitJointInit.l  (*joint.cpRotaryLimitJoint, *a.cpBody, *b.cpBody, min.cpFloat, max.cpFloat) As "_cpRotaryLimitJointInit";  = *cpRotaryLimitJoint
	cpRotaryLimitJointNew.l   (*a.cpBody, *b.cpBody, min.cpFloat, max.cpFloat) As "_cpRotaryLimitJointNew";  = *cpConstraint
	cpRotaryLimitJointSetMax  (*constraint.cpConstraint, max.cpFloat) As "_cpRotaryLimitJointSetMax"
	cpRotaryLimitJointSetMin  (*constraint.cpConstraint, min.cpFloat) As "_cpRotaryLimitJointSetMin"

	cpSegmentShapeAlloc.l   () As "_cpSegmentShapeAlloc";  = *cpSegmentShape
	cpSegmentShapeGetA_pb   (*shape.cpShape, *lp_out.cpVect) As "?cpSegmentShapeGetA_pb@@YAXPBUcpShape@@PAUcpVect@@@Z"
	cpSegmentShapeGetB_pb   (*shape.cpShape, *lp_out.cpVect) As "?cpSegmentShapeGetB_pb@@YAXPBUcpShape@@PAUcpVect@@@Z"
	cpSegmentShapeGetNormal_pb  (*shape.cpShape, *lp_out.cpVect) As "?cpSegmentShapeGetNormal_pb@@YAXPBUcpShape@@PAUcpVect@@@Z"
	cpSegmentShapeGetRadius.cpFloat   (*shape.cpShape) As "_cpSegmentShapeGetRadius"
	cpSegmentShapeInit_pb.l   (*seg.cpSegmentShape, *body.cpBody, *a.cpVect, *b.cpVect, r.cpFloat) As "?cpSegmentShapeInit_pb@@YAPAUcpSegmentShape@@PAU1@PAUcpBody@@PAUcpVect@@2N@Z";  = *cpSegmentShape
	cpSegmentShapeNew_pb.l  (*body.cpBody, *a.cpVect, *b.cpVect, r.cpFloat) As "?cpSegmentShapeNew_pb@@YAPAUcpShape@@PAUcpBody@@PAUcpVect@@1N@Z";  = *cpShape
	cpSegmentShapeSetEndpoints_pb   (*shape.cpShape, *a.cpVect, *b.cpVect) As "?cpSegmentShapeSetEndpoints_pb@@YAXPAUcpShape@@PAUcpVect@@1@Z"
	cpSegmentShapeSetNeighbors_pb   (*shape.cpShape, *prev.cpVect, *next.cpVect) As "?cpSegmentShapeSetNeighbors_pb@@YAXPAUcpShape@@PAUcpVect@@1@Z"
	cpSegmentShapeSetRadius   (*shape.cpShape, radius.cpFloat) As "_cpSegmentShapeSetRadius"

	cpShapeCacheBB_pb   (*shape.cpShape, *lp_out.cpBB) As "?cpShapeCacheBB_pb@@YAXPAUcpShape@@PAUcpBB@@@Z"
	cpShapeDestroy  (*shape.cpShape) As "_cpShapeDestroy"
	cpShapeFree   (*shape.cpShape) As "_cpShapeFree"
	cpShapeGetArea.cpFloat  (*shape.cpShape) As "_cpShapeGetArea"
	cpShapeGetBB_pb   (*shape.cpShape, *lp_out.cpBB) As "?cpShapeGetBB_pb@@YAXPBUcpShape@@PAUcpBB@@@Z"
	cpShapeGetBody.l  (*shape.cpShape) As "_cpShapeGetBody";  = *cpBody
	cpShapeGetCenterOfGravity_pb  (*shape.cpShape, *lp_out.cpVect) As "?cpShapeGetCenterOfGravity_pb@@YAXPAUcpShape@@PAUcpVect@@@Z"
	cpShapeGetCollisionType.cpCollisionType   (*shape.cpShape) As "_cpShapeGetCollisionType"
	cpShapeGetDensity.cpFloat   (*shape.cpShape) As "_cpShapeGetDensity"
	cpShapeGetElasticity.cpFloat  (*shape.cpShape) As "_cpShapeGetElasticity"
	cpShapeGetFilter_pb   (*shape.cpShape, *lp_out.cpShapeFilter) As "?cpShapeGetFilter_pb@@YAXPBUcpShape@@PAUcpShapeFilter@@@Z"
	cpShapeGetFriction.cpFloat  (*shape.cpShape) As "_cpShapeGetFriction"
	cpShapeGetMass.cpFloat  (*shape.cpShape) As "_cpShapeGetMass"
	cpShapeGetMoment.cpFloat  (*shape.cpShape) As "_cpShapeGetMoment"
	cpShapeGetSensor.cpBool   (*shape.cpShape) As "_cpShapeGetSensor"
	cpShapeGetSpace.l   (*shape.cpShape) As "_cpShapeGetSpace";  = *cpSpace
	cpShapeGetSurfaceVelocity_pb  (*shape.cpShape, *lp_out.cpVect) As "?cpShapeGetSurfaceVelocity_pb@@YAXPBUcpShape@@PAUcpVect@@@Z"
	cpShapeGetUserData.l  (*shape.cpShape) As "_cpShapeGetUserData";  = *cpDataPointer
	cpShapeInit_pb.l  (*shape.cpShape, *klass.cpShapeClass, *body.cpBody, *massInfo.cpShapeMassInfo) As "?cpShapeInit_pb@@YAPAUcpShape@@PAU1@PBUcpShapeClass@@PAUcpBody@@PAUcpShapeMassInfo@@@Z";  = *cpShape
	cpShapePointQuery_pb.cpFloat  (*shape.cpShape, *p.cpVect, *info.cpPointQueryInfo) As "?cpShapePointQuery_pb@@YANPBUcpShape@@PAUcpVect@@PAUcpPointQueryInfo@@@Z"
	cpShapesCollide_pb  (*a.cpShape, *b.cpShape, *lp_out.cpContactPointSet) As "?cpShapesCollide_pb@@YAXPBUcpShape@@0PAUcpContactPointSet@@@Z"
	cpShapeSegmentQuery_pb.cpBool   (*shape.cpShape, *a.cpVect, *b.cpVect, radius.cpFloat, *info.cpSegmentQueryInfo) As "?cpShapeSegmentQuery_pb@@YAHPBUcpShape@@PAUcpVect@@1NPAUcpSegmentQueryInfo@@@Z"
	cpShapeSetBody  (*shape.cpShape, *body.cpBody) As "_cpShapeSetBody"
	cpShapeSetCollisionType   (*shape.cpShape, collisionType.cpCollisionType) As "_cpShapeSetCollisionType"
	cpShapeSetDensity   (*shape.cpShape, density.cpFloat) As "_cpShapeSetDensity"
	cpShapeSetElasticity  (*shape.cpShape, elasticity.cpFloat) As "_cpShapeSetElasticity"
	cpShapeSetFilter_pb   (*shape.cpShape, *filter.cpShapeFilter) As "?cpShapeSetFilter_pb@@YAXPAUcpShape@@PAUcpShapeFilter@@@Z"
	cpShapeSetFriction  (*shape.cpShape, friction.cpFloat) As "_cpShapeSetFriction"
	cpShapeSetMass  (*shape.cpShape, mass.cpFloat) As "_cpShapeSetMass"
	cpShapeSetSensor  (*shape.cpShape, sensor.cpBool) As "_cpShapeSetSensor"
	cpShapeSetSurfaceVelocity_pb  (*shape.cpShape, *surfaceVelocity.cpVect) As "?cpShapeSetSurfaceVelocity_pb@@YAXPAUcpShape@@PAUcpVect@@@Z"
	cpShapeSetUserData  (*shape.cpShape, *cpDataPointer) As "_cpShapeSetUserData"
	cpShapeUpdate_pb  (*shape.cpShape, *transform.cpTransform, *lp_out.cpBB) As "?cpShapeUpdate_pb@@YAXPAUcpShape@@PAUcpTransform@@PAUcpBB@@@Z"
	cpShapeUpdateFunc   (*shape.cpShape, *unused) As "?cpShapeUpdateFunc@@YAXPAUcpShape@@PAX@Z"

	cpSimpleMotorAlloc.l  () As "_cpSimpleMotorAlloc";  = *cpSimpleMotor
	cpSimpleMotorGetRate.cpFloat  (*constraint.cpConstraint) As "_cpSimpleMotorGetRate"
	cpSimpleMotorInit.l   (*joint.cpSimpleMotor, *a.cpBody, *b.cpBody, rate.cpFloat) As "_cpSimpleMotorInit";  = *cpSimpleMotor
	cpSimpleMotorNew.l  (*a.cpBody, *b.cpBody, rate.cpFloat) As "_cpSimpleMotorNew";  = *cpConstraint
	cpSimpleMotorSetRate  (*constraint.cpConstraint, rate.cpFloat) As "_cpSimpleMotorSetRate"

	cpSlideJointAlloc.l   () As "_cpSlideJointAlloc";  = *cpSlideJoint
	cpSlideJointGetAnchorA_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpSlideJointGetAnchorA_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpSlideJointGetAnchorB_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpSlideJointGetAnchorB_pb@@YAXPBUcpConstraint@@PAUcpVect@@@Z"
	cpSlideJointGetMax.cpFloat  (*constraint.cpConstraint) As "_cpSlideJointGetMax"
	cpSlideJointGetMin.cpFloat  (*constraint.cpConstraint) As "_cpSlideJointGetMin"
	cpSlideJointInit_pb.l   (*joint.cpSlideJoint, *a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect, min.cpFloat, max.cpFloat) As "?cpSlideJointInit_pb@@YAPAUcpSlideJoint@@PAU1@PAUcpBody@@1PAUcpVect@@2NN@Z";  = *cpSlideJoint
	cpSlideJointNew_pb.l  (*a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect, min.cpFloat, max.cpFloat) As "?cpSlideJointNew_pb@@YAPAUcpConstraint@@PAUcpBody@@0PAUcpVect@@1NN@Z";  = *cpConstraint
	cpSlideJointSetAnchorA_pb   (*constraint.cpConstraint, *anchorA.cpVect) As "?cpSlideJointSetAnchorA_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpSlideJointSetAnchorB_pb   (*constraint.cpConstraint, *anchorB.cpVect) As "?cpSlideJointSetAnchorB_pb@@YAXPAUcpConstraint@@PAUcpVect@@@Z"
	cpSlideJointSetMax  (*constraint.cpConstraint, max.cpFloat) As "_cpSlideJointSetMax"
	cpSlideJointSetMin  (*constraint.cpConstraint, min.cpFloat) As "_cpSlideJointSetMin"

	cpSpaceActivateBody   (*space.cpSpace, *body.cpBody) As "?cpSpaceActivateBody@@YAXPAUcpSpace@@PAUcpBody@@@Z"
	cpSpaceAddBody.l  (*space.cpSpace, *body.cpBody) As "_cpSpaceAddBody";  = *cpBody
	cpSpaceAddCollisionHandler.l  (*space.cpSpace, a.cpCollisionType, b.cpCollisionType) As "_cpSpaceAddCollisionHandler";  = *cpCollisionHandler
	cpSpaceAddConstraint.l  (*space.cpSpace, *constraint.cpConstraint) As "_cpSpaceAddConstraint";  = *cpConstraint
	cpSpaceAddDefaultCollisionHandler.l   (*space.cpSpace) As "_cpSpaceAddDefaultCollisionHandler";  = *cpCollisionHandler
	cpSpaceAddPostStepCallback.cpBool   (*space.cpSpace, *cpPostStepFunc, *key, *data) As "_cpSpaceAddPostStepCallback"
	cpSpaceAddShape.l   (*space.cpSpace, *shape.cpShape) As "_cpSpaceAddShape";  = *cpShape
	cpSpaceAddWildcardHandler.l   (*space.cpSpace, type.cpCollisionType) As "_cpSpaceAddWildcardHandler";  = *cpCollisionHandler
	cpSpaceAlloc.l  () As "_cpSpaceAlloc";  = *cpSpace
	cpSpaceArbiterSetFilter.cpBool  (*arb.cpArbiter, *space.cpSpace) As "?cpSpaceArbiterSetFilter@@YAHPAUcpArbiter@@PAUcpSpace@@@Z"
	cpSpaceBBQuery_pb   (*space.cpSpace, *bb.cpBB, *filter.cpShapeFilter, *cpSpaceBBQueryFunc, *data) As "?cpSpaceBBQuery_pb@@YAXPAUcpSpace@@PAUcpBB@@PAUcpShapeFilter@@P6AXPAUcpShape@@PAX@Z4@Z"
	cpSpaceCollideShapes.cpCollisionID  (*a.cpShape, *b.cpShape, id.cpCollisionID, *space.cpSpace) As "?cpSpaceCollideShapes@@YAIPAUcpShape@@0IPAUcpSpace@@@Z"
	cpSpaceContainsBody.cpBool  (*space.cpSpace, *body.cpBody) As "_cpSpaceContainsBody"
	cpSpaceContainsConstraint.cpBool  (*space.cpSpace, *constraint.cpConstraint) As "_cpSpaceContainsConstraint"
	cpSpaceContainsShape.cpBool   (*space.cpSpace, *shape.cpShape) As "_cpSpaceContainsShape"
; 	cpSpaceDebugDraw  (*space.cpSpace, *options.cpSpaceDebugDrawOptions) As "_cpSpaceDebugDraw"
	cpSpaceDestroy  (*space.cpSpace) As "_cpSpaceDestroy"
	cpSpaceEachBody   (*space.cpSpace, *cpSpaceBodyIteratorFunc, *data) As "_cpSpaceEachBody"
	cpSpaceEachConstraint   (*space.cpSpace, *cpSpaceConstraintIteratorFunc, *data) As "_cpSpaceEachConstraint"
	cpSpaceEachShape  (*space.cpSpace, *cpSpaceShapeIteratorFunc, *data) As "_cpSpaceEachShape"
	cpSpaceFilterArbiters   (*space.cpSpace, *body.cpBody, *filter.cpShape) As "?cpSpaceFilterArbiters@@YAXPAUcpSpace@@PAUcpBody@@PAUcpShape@@@Z"
	cpSpaceFree   (*space.cpSpace) As "_cpSpaceFree"
	cpSpaceGetCollisionBias.cpFloat   (*space.cpSpace) As "_cpSpaceGetCollisionBias"
	cpSpaceGetCollisionPersistence.cpTimestamp  (*space.cpSpace) As "_cpSpaceGetCollisionPersistence"
	cpSpaceGetCollisionSlop.cpFloat   (*space.cpSpace) As "_cpSpaceGetCollisionSlop"
	cpSpaceGetCurrentTimeStep.cpFloat   (*space.cpSpace) As "_cpSpaceGetCurrentTimeStep"
	cpSpaceGetDamping.cpFloat   (*space.cpSpace) As "_cpSpaceGetDamping"
	cpSpaceGetGravity_pb  (*space.cpSpace, *lp_out.cpVect) As "?cpSpaceGetGravity_pb@@YAXPBUcpSpace@@PAUcpVect@@@Z"
	cpSpaceGetIdleSpeedThreshold.cpFloat  (*space.cpSpace) As "_cpSpaceGetIdleSpeedThreshold"
	cpSpaceGetIterations.l  (*space.cpSpace) As "_cpSpaceGetIterations"
	cpSpaceGetPostStepCallback.l  (*space.cpSpace, *key) As "?cpSpaceGetPostStepCallback@@YAPAUcpPostStepCallback@@PAUcpSpace@@PAX@Z";  = *cpPostStepCallback
	cpSpaceGetSleepTimeThreshold.cpFloat  (*space.cpSpace) As "_cpSpaceGetSleepTimeThreshold"
	cpSpaceGetStaticBody.l  (*space.cpSpace) As "_cpSpaceGetStaticBody";  = *cpBody
	cpSpaceGetUserData.l  (*space.cpSpace) As "_cpSpaceGetUserData";  = *cpDataPointer
	cpSpaceHashAlloc.l  () As "_cpSpaceHashAlloc";  = *cpSpaceHash
	cpSpaceHashInit.l   (*hash.cpSpaceHash, celldim.cpFloat, numcells.l, *cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "_cpSpaceHashInit";  = *cpSpatialIndex
	cpSpaceHashNew.l  (celldim.cpFloat, cells.l, *cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "_cpSpaceHashNew";  = *cpSpatialIndex
	cpSpaceHashResize   (*hash.cpSpaceHash, celldim.cpFloat, numcells.l) As "_cpSpaceHashResize"
	cpSpaceInit.l   (*space.cpSpace) As "_cpSpaceInit";  = *cpSpace
	cpSpaceIsLocked.cpBool  (*space.cpSpace) As "_cpSpaceIsLocked"
	cpSpaceLock   (*space.cpSpace) As "?cpSpaceLock@@YAXPAUcpSpace@@@Z"
	cpSpaceNew.l  () As "_cpSpaceNew";  = *cpSpace
	cpSpacePointQuery_pb  (*space.cpSpace, *point.cpVect, maxDistance.cpFloat, *filter.cpShapeFilter, *cpSpacePointQueryFunc, *data) As "?cpSpacePointQuery_pb@@YAXPAUcpSpace@@PAUcpVect@@NPAUcpShapeFilter@@P6AXPAUcpShape@@1N1PAX@Z4@Z"
	cpSpacePointQueryNearest_pb.l   (*space.cpSpace, *point.cpVect, maxDistance.cpFloat, *filter.cpShapeFilter, *out.cpPointQueryInfo) As "?cpSpacePointQueryNearest_pb@@YAPAUcpShape@@PAUcpSpace@@PAUcpVect@@NPAUcpShapeFilter@@PAUcpPointQueryInfo@@@Z";  = *cpShape
	cpSpaceProcessComponents  (*space.cpSpace, dt.cpFloat) As "?cpSpaceProcessComponents@@YAXPAUcpSpace@@N@Z"
	cpSpacePushContacts   (*space.cpSpace, count.l) As "?cpSpacePushContacts@@YAXPAUcpSpace@@H@Z"
	cpSpacePushFreshContactBuffer   (*space.cpSpace) As "?cpSpacePushFreshContactBuffer@@YAXPAUcpSpace@@@Z"
	cpSpaceReindexShape   (*space.cpSpace, *shape.cpShape) As "_cpSpaceReindexShape"
	cpSpaceReindexShapesForBody   (*space.cpSpace, *body.cpBody) As "_cpSpaceReindexShapesForBody"
	cpSpaceReindexStatic  (*space.cpSpace) As "_cpSpaceReindexStatic"
	cpSpaceRemoveBody   (*space.cpSpace, *body.cpBody) As "_cpSpaceRemoveBody"
	cpSpaceRemoveConstraint   (*space.cpSpace, *constraint.cpConstraint) As "_cpSpaceRemoveConstraint"
	cpSpaceRemoveShape  (*space.cpSpace, *shape.cpShape) As "_cpSpaceRemoveShape"
	cpSpaceSegmentQuery_pb  (*space.cpSpace, *start.cpVect, *end.cpVect, radius.cpFloat, *filter.cpShapeFilter, *cpSpaceSegmentQueryFunc, *data) As "?cpSpaceSegmentQuery_pb@@YAXPAUcpSpace@@PAUcpVect@@1NPAUcpShapeFilter@@P6AXPAUcpShape@@11NPAX@Z4@Z"
	cpSpaceSegmentQueryFirst_pb.l   (*space.cpSpace, *start.cpVect, *end.cpVect, radius.cpFloat, *filter.cpShapeFilter, *out.cpSegmentQueryInfo) As "?cpSpaceSegmentQueryFirst_pb@@YAPAUcpShape@@PAUcpSpace@@PAUcpVect@@1NPAUcpShapeFilter@@PAUcpSegmentQueryInfo@@@Z";  = *cpShape
	cpSpaceSetCollisionBias   (*space.cpSpace, collisionBias.cpFloat) As "_cpSpaceSetCollisionBias"
	cpSpaceSetCollisionPersistence  (*space.cpSpace, collisionPersistence.cpTimestamp) As "_cpSpaceSetCollisionPersistence"
	cpSpaceSetCollisionSlop   (*space.cpSpace, collisionSlop.cpFloat) As "_cpSpaceSetCollisionSlop"
	cpSpaceSetDamping   (*space.cpSpace, damping.cpFloat) As "_cpSpaceSetDamping"
	cpSpaceSetGravity_pb  (*space.cpSpace, *gravity.cpVect) As "?cpSpaceSetGravity_pb@@YAXPAUcpSpace@@PAUcpVect@@@Z"
	cpSpaceSetIdleSpeedThreshold  (*space.cpSpace, idleSpeedThreshold.cpFloat) As "_cpSpaceSetIdleSpeedThreshold"
	cpSpaceSetIterations  (*space.cpSpace, iterations.l) As "_cpSpaceSetIterations"
	cpSpaceSetSleepTimeThreshold  (*space.cpSpace, sleepTimeThreshold.cpFloat) As "_cpSpaceSetSleepTimeThreshold"
	cpSpaceSetStaticBody  (*space.cpSpace, *body.cpBody) As "?cpSpaceSetStaticBody@@YAXPAUcpSpace@@PAUcpBody@@@Z"
	cpSpaceSetUserData  (*space.cpSpace, *cpDataPointer) As "_cpSpaceSetUserData"
	cpSpaceShapeQuery.cpBool  (*space.cpSpace, *shape.cpShape, *cpSpaceShapeQueryFunc, *data) As "_cpSpaceShapeQuery"
	cpSpaceStep   (*space.cpSpace, dt.cpFloat) As "_cpSpaceStep"
	cpSpaceUnlock   (*space.cpSpace, runPostStep.cpBool) As "?cpSpaceUnlock@@YAXPAUcpSpace@@H@Z"
	cpSpaceUseSpatialHash   (*space.cpSpace, cDim.cpFloat, count.l) As "_cpSpaceUseSpatialHash"

	cpSpatialIndexCollideStatic   (*dynamicIndex.cpSpatialIndex, *staticIndex.cpSpatialIndex, *cpSpatialIndexQueryFunc, *data) As "_cpSpatialIndexCollideStatic"
	cpSpatialIndexFree  (*index.cpSpatialIndex) As "_cpSpatialIndexFree"
	cpSpatialIndexInit.l  (*index.cpSpatialIndex, *klass.cpSpatialIndexClass, *cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "?cpSpatialIndexInit@@YAPAUcpSpatialIndex@@PAU1@PAUcpSpatialIndexClass@@P6A?AUcpBB@@PAX@Z0@Z";  = *cpSpatialIndex

	cpSweep1DAlloc.l  () As "_cpSweep1DAlloc";  = *cpSweep1D
	cpSweep1DInit.l   (*sweep.cpSweep1D, *cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "_cpSweep1DInit";  = *cpSpatialIndex
	cpSweep1DNew.l  (*cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "_cpSweep1DNew";  = *cpSpatialIndex
EndImport
CompilerElseIf #PB_Compiler_Processor = #PB_Processor_x64
ImportC ".\lib\chipmunk\chipmunk.7.0.1_x64.lib"
	cpArbiterApplyCachedImpulse   (*arb.cpArbiter, dt_coef.cpFloat) As "?cpArbiterApplyCachedImpulse@@YAXPEAUcpArbiter@@N@Z"
	cpArbiterApplyImpulse   (*arb.cpArbiter) As "?cpArbiterApplyImpulse@@YAXPEAUcpArbiter@@@Z"
	cpArbiterCallWildcardBeginA.cpBool  (*arb.cpArbiter, *space.cpSpace) As "cpArbiterCallWildcardBeginA"
	cpArbiterCallWildcardBeginB.cpBool  (*arb.cpArbiter, *space.cpSpace) As "cpArbiterCallWildcardBeginB"
	cpArbiterCallWildcardPostSolveA   (*arb.cpArbiter, *space.cpSpace) As "cpArbiterCallWildcardPostSolveA"
	cpArbiterCallWildcardPostSolveB   (*arb.cpArbiter, *space.cpSpace) As "cpArbiterCallWildcardPostSolveB"
	cpArbiterCallWildcardPreSolveA.cpBool   (*arb.cpArbiter, *space.cpSpace) As "cpArbiterCallWildcardPreSolveA"
	cpArbiterCallWildcardPreSolveB.cpBool   (*arb.cpArbiter, *space.cpSpace) As "cpArbiterCallWildcardPreSolveB"
	cpArbiterCallWildcardSeparateA  (*arb.cpArbiter, *space.cpSpace) As "cpArbiterCallWildcardSeparateA"
	cpArbiterCallWildcardSeparateB  (*arb.cpArbiter, *space.cpSpace) As "cpArbiterCallWildcardSeparateB"
	cpArbiterGetBodies  (*arb.cpArbiter, *a.cpBody, *b.cpBody) As "cpArbiterGetBodies"
	cpArbiterGetContactPointSet_pb  (*arb.cpArbiter, *lp_out.cpContactPointSet) As "?cpArbiterGetContactPointSet_pb@@YAXPEBUcpArbiter@@PEAUcpContactPointSet@@@Z"
	cpArbiterGetCount.l   (*arb.cpArbiter) As "cpArbiterGetCount"
	cpArbiterGetDepth.cpFloat   (*arb.cpArbiter, i.l) As "cpArbiterGetDepth"
	cpArbiterGetFriction.cpFloat  (*arb.cpArbiter) As "cpArbiterGetFriction"
	cpArbiterGetNormal_pb   (*arb.cpArbiter, *lp_out.cpVect) As "?cpArbiterGetNormal_pb@@YAXPEBUcpArbiter@@PEAUcpVect@@@Z"
	cpArbiterGetPointA_pb   (*arb.cpArbiter, i.l, *lp_out.cpVect) As "?cpArbiterGetPointA_pb@@YAXPEBUcpArbiter@@HPEAUcpVect@@@Z"
	cpArbiterGetPointB_pb   (*arb.cpArbiter, i.l, *lp_out.cpVect) As "?cpArbiterGetPointB_pb@@YAXPEBUcpArbiter@@HPEAUcpVect@@@Z"
	cpArbiterGetRestitution.cpFloat   (*arb.cpArbiter) As "cpArbiterGetRestitution"
	cpArbiterGetShapes  (*arb.cpArbiter, *a.cpShape, *b.cpShape) As "cpArbiterGetShapes"
	cpArbiterGetSurfaceVelocity_pb  (*arb.cpArbiter, *lp_out.cpVect) As "?cpArbiterGetSurfaceVelocity_pb@@YAXPEAUcpArbiter@@PEAUcpVect@@@Z"
	cpArbiterGetUserData.l  (*arb.cpArbiter) As "cpArbiterGetUserData";  = *cpDataPointer
	cpArbiterIgnore.cpBool  (*arb.cpArbiter) As "cpArbiterIgnore"
	cpArbiterInit.l   (*arb.cpArbiter, *a.cpShape, *b.cpShape) As "?cpArbiterInit@@YAPEAUcpArbiter@@PEAU1@PEAUcpShape@@1@Z";  = *cpArbiter
	cpArbiterIsFirstContact.cpBool  (*arb.cpArbiter) As "cpArbiterIsFirstContact"
	cpArbiterIsRemoval.cpBool   (*arb.cpArbiter) As "cpArbiterIsRemoval"
	cpArbiterPreStep  (*arb.cpArbiter, dt.cpFloat, slop.cpFloat, bias.cpFloat) As "?cpArbiterPreStep@@YAXPEAUcpArbiter@@NNN@Z"
	cpArbiterSetContactPointSet   (*arb.cpArbiter, *set.cpContactPointSet) As "cpArbiterSetContactPointSet"
	cpArbiterSetFriction  (*arb.cpArbiter, friction.cpFloat) As "cpArbiterSetFriction"
	cpArbiterSetRestitution   (*arb.cpArbiter, restitution.cpFloat) As "cpArbiterSetRestitution"
	cpArbiterSetSurfaceVelocity_pb  (*arb.cpArbiter, *vr.cpVect) As "?cpArbiterSetSurfaceVelocity_pb@@YAXPEAUcpArbiter@@PEAUcpVect@@@Z"
	cpArbiterSetUserData  (*arb.cpArbiter, *cpDataPointer) As "cpArbiterSetUserData"
	cpArbiterTotalImpulse_pb  (*arb.cpArbiter, *lp_out.cpVect) As "?cpArbiterTotalImpulse_pb@@YAXPEBUcpArbiter@@PEAUcpVect@@@Z"
	cpArbiterTotalKE.cpFloat  (*arb.cpArbiter) As "cpArbiterTotalKE"
	cpArbiterUnthread   (*arb.cpArbiter) As "?cpArbiterUnthread@@YAXPEAUcpArbiter@@@Z"
	cpArbiterUpdate   (*arb.cpArbiter, *info.cpCollisionInfo, *space.cpSpace) As "?cpArbiterUpdate@@YAXPEAUcpArbiter@@PEAUcpCollisionInfo@@PEAUcpSpace@@@Z"

	cpAreaForCircle.cpFloat   (r1.cpFloat, r2.cpFloat) As "cpAreaForCircle"
	cpAreaForPoly.cpFloat   (count.l, *verts.cpVect, r.cpFloat) As "cpAreaForPoly"
	cpAreaForSegment_pb.cpFloat   (*a.cpVect, *b.cpVect, r.cpFloat) As "?cpAreaForSegment_pb@@YANPEAUcpVect@@0N@Z"

	cpArrayContains.cpBool  (*arr.cpArray, *ptr) As "?cpArrayContains@@YAHPEAUcpArray@@PEAX@Z"
	cpArrayDeleteObj  (*arr.cpArray, *obj) As "?cpArrayDeleteObj@@YAXPEAUcpArray@@PEAX@Z"
	cpArrayFree   (*arr.cpArray) As "?cpArrayFree@@YAXPEAUcpArray@@@Z"
	cpArrayFreeEach   (*arr.cpArray, *freeFunc) As "?cpArrayFreeEach@@YAXPEAUcpArray@@P6AXPEAX@Z@Z"
	cpArrayNew.l  (size.l) As "?cpArrayNew@@YAPEAUcpArray@@H@Z";  = *cpArray
	cpArrayPop.l  (*arr.cpArray) As "?cpArrayPop@@YAPEAXPEAUcpArray@@@Z";  = *void
	cpArrayPush   (*arr.cpArray, *object) As "?cpArrayPush@@YAXPEAUcpArray@@PEAX@Z"

	cpBBTreeAlloc.l   () As "cpBBTreeAlloc";  = *cpBBTree
	cpBBTreeInit.l  (*tree.cpBBTree, *cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "cpBBTreeInit";  = *cpSpatialIndex
	cpBBTreeNew.l   (*cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "cpBBTreeNew";  = *cpSpatialIndex
	cpBBTreeOptimize  (*index.cpSpatialIndex) As "cpBBTreeOptimize"
	cpBBTreeSetVelocityFunc   (*index.cpSpatialIndex, *cpBBTreeVelocityFunc) As "cpBBTreeSetVelocityFunc"

	cpBodyAccumulateMassFromShapes  (*body.cpBody) As "?cpBodyAccumulateMassFromShapes@@YAXPEAUcpBody@@@Z"
	cpBodyActivate  (*body.cpBody) As "cpBodyActivate"
	cpBodyActivateStatic  (*body.cpBody, *filter.cpShape) As "cpBodyActivateStatic"
	cpBodyAddShape  (*body.cpBody, *shape.cpShape) As "?cpBodyAddShape@@YAXPEAUcpBody@@PEAUcpShape@@@Z"
	cpBodyAlloc.l   () As "cpBodyAlloc";  = *cpBody
	cpBodyApplyForceAtLocalPoint_pb   (*body.cpBody, *force.cpVect, *point.cpVect) As "?cpBodyApplyForceAtLocalPoint_pb@@YAXPEAUcpBody@@PEAUcpVect@@1@Z"
	cpBodyApplyForceAtWorldPoint_pb   (*body.cpBody, *force.cpVect, *point.cpVect) As "?cpBodyApplyForceAtWorldPoint_pb@@YAXPEAUcpBody@@PEAUcpVect@@1@Z"
	cpBodyApplyImpulseAtLocalPoint_pb   (*body.cpBody, *impulse.cpVect, *point.cpVect) As "?cpBodyApplyImpulseAtLocalPoint_pb@@YAXPEAUcpBody@@PEAUcpVect@@1@Z"
	cpBodyApplyImpulseAtWorldPoint_pb   (*body.cpBody, *impulse.cpVect, *point.cpVect) As "?cpBodyApplyImpulseAtWorldPoint_pb@@YAXPEAUcpBody@@PEAUcpVect@@1@Z"
	cpBodyDestroy   (*body.cpBody) As "cpBodyDestroy"
	cpBodyEachArbiter   (*body.cpBody, *cpBodyArbiterIteratorFunc, *data) As "cpBodyEachArbiter"
	cpBodyEachConstraint  (*body.cpBody, *cpBodyConstraintIteratorFunc, *data) As "cpBodyEachConstraint"
	cpBodyEachShape   (*body.cpBody, *cpBodyShapeIteratorFunc, *data) As "cpBodyEachShape"
	cpBodyFree  (*body.cpBody) As "cpBodyFree"
	cpBodyGetAngle.cpFloat  (*body.cpBody) As "cpBodyGetAngle"
	cpBodyGetAngularVelocity.cpFloat  (*body.cpBody) As "cpBodyGetAngularVelocity"
	cpBodyGetCenterOfGravity_pb   (*body.cpBody, *lp_out.cpVect) As "?cpBodyGetCenterOfGravity_pb@@YAXPEBUcpBody@@PEAUcpVect@@@Z"
	cpBodyGetForce_pb   (*body.cpBody, *lp_out.cpVect) As "?cpBodyGetForce_pb@@YAXPEBUcpBody@@PEAUcpVect@@@Z"
	cpBodyGetMass.cpFloat   (*body.cpBody) As "cpBodyGetMass"
	cpBodyGetMoment.cpFloat   (*body.cpBody) As "cpBodyGetMoment"
	cpBodyGetPosition_pb  (*body.cpBody, *lp_out.cpVect) As "?cpBodyGetPosition_pb@@YAXPEBUcpBody@@PEAUcpVect@@@Z"
	cpBodyGetRotation_pb  (*body.cpBody, *lp_out.cpVect) As "?cpBodyGetRotation_pb@@YAXPEBUcpBody@@PEAUcpVect@@@Z"
	cpBodyGetSpace.l  (*body.cpBody) As "cpBodyGetSpace";  = *cpSpace
	cpBodyGetTorque.cpFloat   (*body.cpBody) As "cpBodyGetTorque"
	cpBodyGetType.cpBodyType  (*body.cpBody) As "cpBodyGetType"
	cpBodyGetUserData.l   (*body.cpBody) As "cpBodyGetUserData";  = *cpDataPointer
	cpBodyGetVelocity_pb  (*body.cpBody, *lp_out.cpVect) As "?cpBodyGetVelocity_pb@@YAXPEBUcpBody@@PEAUcpVect@@@Z"
	cpBodyGetVelocityAtLocalPoint_pb  (*body.cpBody, *point.cpVect, *lp_out.cpVect) As "?cpBodyGetVelocityAtLocalPoint_pb@@YAXPEBUcpBody@@PEAUcpVect@@1@Z"
	cpBodyGetVelocityAtWorldPoint_pb  (*body.cpBody, *point.cpVect, *lp_out.cpVect) As "?cpBodyGetVelocityAtWorldPoint_pb@@YAXPEBUcpBody@@PEAUcpVect@@1@Z"
	cpBodyInit.l  (*body.cpBody, mass.cpFloat, moment.cpFloat) As "cpBodyInit";  = *cpBody
	cpBodyIsSleeping.cpBool   (*body.cpBody) As "cpBodyIsSleeping"
	cpBodyKineticEnergy.cpFloat   (*body.cpBody) As "cpBodyKineticEnergy"
	cpBodyLocalToWorld_pb   (*body.cpBody, *point.cpVect, *lp_out.cpVect) As "?cpBodyLocalToWorld_pb@@YAXPEBUcpBody@@PEBUcpVect@@PEAU2@@Z"
	cpBodyNew.l   (mass.cpFloat, moment.cpFloat) As "cpBodyNew";  = *cpBody
	cpBodyNewKinematic.l  () As "cpBodyNewKinematic";  = *cpBody
	cpBodyNewStatic.l   () As "cpBodyNewStatic";  = *cpBody
	cpBodyRemoveConstraint  (*body.cpBody, *constraint.cpConstraint) As "?cpBodyRemoveConstraint@@YAXPEAUcpBody@@PEAUcpConstraint@@@Z"
	cpBodyRemoveShape   (*body.cpBody, *shape.cpShape) As "?cpBodyRemoveShape@@YAXPEAUcpBody@@PEAUcpShape@@@Z"
	cpBodySetAngle  (*body.cpBody, angle.cpFloat) As "cpBodySetAngle"
	cpBodySetAngularVelocity  (*body.cpBody, angularVelocity.cpFloat) As "cpBodySetAngularVelocity"
	cpBodySetCenterOfGravity_pb   (*body.cpBody, *cog.cpVect) As "?cpBodySetCenterOfGravity_pb@@YAXPEAUcpBody@@PEAUcpVect@@@Z"
	cpBodySetForce_pb   (*body.cpBody, *force.cpVect) As "?cpBodySetForce_pb@@YAXPEAUcpBody@@PEAUcpVect@@@Z"
	cpBodySetMass   (*body.cpBody, mass.cpFloat) As "cpBodySetMass"
	cpBodySetMoment   (*body.cpBody, moment.cpFloat) As "cpBodySetMoment"
	cpBodySetPosition_pb  (*body.cpBody, *position.cpVect) As "?cpBodySetPosition_pb@@YAXPEAUcpBody@@PEAUcpVect@@@Z"
	cpBodySetPositionUpdateFunc   (*body.cpBody, *cpBodyPositionFunc) As "cpBodySetPositionUpdateFunc"
	cpBodySetTorque   (*body.cpBody, torque.cpFloat) As "cpBodySetTorque"
	cpBodySetType   (*body.cpBody, type.cpBodyType) As "cpBodySetType"
	cpBodySetUserData   (*body.cpBody, *cpDataPointer) As "cpBodySetUserData"
	cpBodySetVelocity_pb  (*body.cpBody, *velocity.cpVect) As "?cpBodySetVelocity_pb@@YAXPEAUcpBody@@PEAUcpVect@@@Z"
	cpBodySetVelocityUpdateFunc   (*body.cpBody, *cpBodyVelocityFunc) As "cpBodySetVelocityUpdateFunc"
	cpBodySleep   (*body.cpBody) As "cpBodySleep"
	cpBodySleepWithGroup  (*body.cpBody, *group.cpBody) As "cpBodySleepWithGroup"
	cpBodyUpdatePosition  (*body.cpBody, dt.cpFloat) As "cpBodyUpdatePosition"
	cpBodyUpdateVelocity_pb   (*body.cpBody, *gravity.cpVect, damping.cpFloat, dt.cpFloat) As "?cpBodyUpdateVelocity_pb@@YAXPEAUcpBody@@PEAUcpVect@@NN@Z"
	cpBodyWorldToLocal_pb   (*body.cpBody, *point.cpVect, *lp_out.cpVect) As "?cpBodyWorldToLocal_pb@@YAXPEBUcpBody@@PEBUcpVect@@PEAU2@@Z"

	cpBoxShapeInit.l  (*poly.cpPolyShape, *body.cpBody, width.cpFloat, height.cpFloat, radius.cpFloat) As "cpBoxShapeInit";  = *cpPolyShape
	cpBoxShapeInit2_pb.l  (*poly.cpPolyShape, *body.cpBody, *box.cpBB, radius.cpFloat) As "?cpBoxShapeInit2_pb@@YAPEAUcpPolyShape@@PEAU1@PEAUcpBody@@PEAUcpBB@@N@Z";  = *cpPolyShape
	cpBoxShapeNew.l   (*body.cpBody, width.cpFloat, height.cpFloat, radius.cpFloat) As "cpBoxShapeNew";  = *cpShape
	cpBoxShapeNew2_pb.l   (*body.cpBody, *box.cpBB, radius.cpFloat) As "?cpBoxShapeNew2_pb@@YAPEAUcpShape@@PEAUcpBody@@PEAUcpBB@@N@Z";  = *cpShape

	cpCentroidForPoly_pb  (count.l, *verts.cpVect, *lp_out.cpVect) As "?cpCentroidForPoly_pb@@YAXHPEBUcpVect@@PEAU1@@Z"
	cpCheckSignedArea_pb.cpBool   (*a.cpVect, *b.cpVect, *c.cpVect) As "?cpCheckSignedArea_pb@@YAHPEBUcpVect@@00@Z"

	cpCircleShapeAlloc.l  () As "cpCircleShapeAlloc";  = *cpCircleShape
	cpCircleShapeGetOffset_pb   (*shape.cpShape, *lp_out.cpVect) As "?cpCircleShapeGetOffset_pb@@YAXPEBUcpShape@@PEAUcpVect@@@Z"
	cpCircleShapeGetRadius.cpFloat  (*shape.cpShape) As "cpCircleShapeGetRadius"
	cpCircleShapeInit_pb.l  (*circle.cpCircleShape, *body.cpBody, radius.cpFloat, *offset.cpVect) As "?cpCircleShapeInit_pb@@YAPEAUcpCircleShape@@PEAU1@PEAUcpBody@@NPEAUcpVect@@@Z";  = *cpCircleShape
	cpCircleShapeNew_pb.l   (*body.cpBody, radius.cpFloat, *offset.cpVect) As "?cpCircleShapeNew_pb@@YAPEAUcpShape@@PEAUcpBody@@NPEAUcpVect@@@Z";  = *cpShape
	cpCircleShapeSetOffset_pb   (*shape.cpShape, *offset.cpVect) As "?cpCircleShapeSetOffset_pb@@YAXPEAUcpShape@@PEAUcpVect@@@Z"
	cpCircleShapeSetRadius  (*shape.cpShape, radius.cpFloat) As "cpCircleShapeSetRadius"

	cpCollide_pb  (*a.cpShape, *b.cpShape, id.cpCollisionID, *contacts.cpContact, *lp_out.cpCollisionInfo) As "?cpCollide_pb@@YAXPEBUcpShape@@0IPEAUcpContact@@PEAUcpCollisionInfo@@@Z"

	cpConstraintDestroy   (*constraint.cpConstraint) As "cpConstraintDestroy"
	cpConstraintFree  (*constraint.cpConstraint) As "cpConstraintFree"
	cpConstraintGetBodyA.l  (*constraint.cpConstraint) As "cpConstraintGetBodyA";  = *cpBody
	cpConstraintGetBodyB.l  (*constraint.cpConstraint) As "cpConstraintGetBodyB";  = *cpBody
	cpConstraintGetCollideBodies.cpBool   (*constraint.cpConstraint) As "cpConstraintGetCollideBodies"
	cpConstraintGetErrorBias.cpFloat  (*constraint.cpConstraint) As "cpConstraintGetErrorBias"
	cpConstraintGetImpulse.cpFloat  (*constraint.cpConstraint) As "cpConstraintGetImpulse"
	cpConstraintGetMaxBias.cpFloat  (*constraint.cpConstraint) As "cpConstraintGetMaxBias"
	cpConstraintGetMaxForce.cpFloat   (*constraint.cpConstraint) As "cpConstraintGetMaxForce"
	cpConstraintGetPostSolveFunc.l  (*constraint.cpConstraint) As "cpConstraintGetPostSolveFunc";  = cpConstraintPostSolveFunc
	cpConstraintGetPreSolveFunc.l   (*constraint.cpConstraint) As "cpConstraintGetPreSolveFunc";  = cpConstraintPreSolveFunc
	cpConstraintGetSpace.l  (*constraint.cpConstraint) As "cpConstraintGetSpace";  = *cpSpace
	cpConstraintGetUserData.l   (*constraint.cpConstraint) As "cpConstraintGetUserData";  = *cpDataPointer
	cpConstraintInit  (*constraint.cpConstraint, *klass.cpConstraintClass, *a.cpBody, *b.cpBody) As "?cpConstraintInit@@YAXPEAUcpConstraint@@PEBUcpConstraintClass@@PEAUcpBody@@2@Z"
	cpConstraintIsDampedRotarySpring.cpBool   (*constraint.cpConstraint) As "cpConstraintIsDampedRotarySpring"
	cpConstraintIsDampedSpring.cpBool   (*constraint.cpConstraint) As "cpConstraintIsDampedSpring"
	cpConstraintIsGearJoint.cpBool  (*constraint.cpConstraint) As "cpConstraintIsGearJoint"
	cpConstraintIsGrooveJoint.cpBool  (*constraint.cpConstraint) As "cpConstraintIsGrooveJoint"
	cpConstraintIsPinJoint.cpBool   (*constraint.cpConstraint) As "cpConstraintIsPinJoint"
	cpConstraintIsPivotJoint.cpBool   (*constraint.cpConstraint) As "cpConstraintIsPivotJoint"
	cpConstraintIsRatchetJoint.cpBool   (*constraint.cpConstraint) As "cpConstraintIsRatchetJoint"
	cpConstraintIsRotaryLimitJoint.cpBool   (*constraint.cpConstraint) As "cpConstraintIsRotaryLimitJoint"
	cpConstraintIsSimpleMotor.cpBool  (*constraint.cpConstraint) As "cpConstraintIsSimpleMotor"
	cpConstraintIsSlideJoint.cpBool   (*constraint.cpConstraint) As "cpConstraintIsSlideJoint"
	cpConstraintSetCollideBodies  (*constraint.cpConstraint, collideBodies.cpBool) As "cpConstraintSetCollideBodies"
	cpConstraintSetErrorBias  (*constraint.cpConstraint, errorBias.cpFloat) As "cpConstraintSetErrorBias"
	cpConstraintSetMaxBias  (*constraint.cpConstraint, maxBias.cpFloat) As "cpConstraintSetMaxBias"
	cpConstraintSetMaxForce   (*constraint.cpConstraint, maxForce.cpFloat) As "cpConstraintSetMaxForce"
	cpConstraintSetPostSolveFunc  (*constraint.cpConstraint, *cpConstraintPostSolveFunc) As "cpConstraintSetPostSolveFunc"
	cpConstraintSetPreSolveFunc   (*constraint.cpConstraint, *cpConstraintPreSolveFunc) As "cpConstraintSetPreSolveFunc"
	cpConstraintSetUserData   (*constraint.cpConstraint, *cpDataPointer) As "cpConstraintSetUserData"

	cpContactBufferGetArray.l   (*space.cpSpace) As "?cpContactBufferGetArray@@YAPEAUcpContact@@PEAUcpSpace@@@Z";  = *cpContact
	cpConvexHull.l  (count.l, *verts.cpVect, *result.cpVect, *first.Long, tol.cpFloat) As "cpConvexHull"

	cpDampedRotarySpringAlloc.l   () As "cpDampedRotarySpringAlloc";  = *cpDampedRotarySpring
	cpDampedRotarySpringGetDamping.cpFloat  (*constraint.cpConstraint) As "cpDampedRotarySpringGetDamping"
	cpDampedRotarySpringGetRestAngle.cpFloat  (*constraint.cpConstraint) As "cpDampedRotarySpringGetRestAngle"
	cpDampedRotarySpringGetSpringTorqueFunc.l   (*constraint.cpConstraint) As "cpDampedRotarySpringGetSpringTorqueFunc";  = cpDampedRotarySpringTorqueFunc
	cpDampedRotarySpringGetStiffness.cpFloat  (*constraint.cpConstraint) As "cpDampedRotarySpringGetStiffness"
	cpDampedRotarySpringInit.l  (*spring.cpDampedRotarySpring, *a.cpBody, *b.cpBody, restAngle.cpFloat, stiffness.cpFloat, damping.cpFloat) As "cpDampedRotarySpringInit";  = *cpDampedRotarySpring
	cpDampedRotarySpringNew.l   (*a.cpBody, *b.cpBody, restAngle.cpFloat, stiffness.cpFloat, damping.cpFloat) As "cpDampedRotarySpringNew";  = *cpConstraint
	cpDampedRotarySpringSetDamping  (*constraint.cpConstraint, damping.cpFloat) As "cpDampedRotarySpringSetDamping"
	cpDampedRotarySpringSetRestAngle  (*constraint.cpConstraint, restAngle.cpFloat) As "cpDampedRotarySpringSetRestAngle"
	cpDampedRotarySpringSetSpringTorqueFunc   (*constraint.cpConstraint, *cpDampedRotarySpringTorqueFunc) As "cpDampedRotarySpringSetSpringTorqueFunc"
	cpDampedRotarySpringSetStiffness  (*constraint.cpConstraint, stiffness.cpFloat) As "cpDampedRotarySpringSetStiffness"

	cpDampedSpringAlloc.l   () As "cpDampedSpringAlloc";  = *cpDampedSpring
	cpDampedSpringGetAnchorA_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpDampedSpringGetAnchorA_pb@@YAXPEBUcpConstraint@@PEAUcpVect@@@Z"
	cpDampedSpringGetAnchorB_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpDampedSpringGetAnchorB_pb@@YAXPEBUcpConstraint@@PEAUcpVect@@@Z"
	cpDampedSpringGetDamping.cpFloat  (*constraint.cpConstraint) As "cpDampedSpringGetDamping"
	cpDampedSpringGetRestLength.cpFloat   (*constraint.cpConstraint) As "cpDampedSpringGetRestLength"
	cpDampedSpringGetSpringForceFunc.l  (*constraint.cpConstraint) As "cpDampedSpringGetSpringForceFunc";  = cpDampedSpringForceFunc
	cpDampedSpringGetStiffness.cpFloat  (*constraint.cpConstraint) As "cpDampedSpringGetStiffness"
	cpDampedSpringInit_pb.l   (*spring.cpDampedSpring, *a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect, restLength.cpFloat, stiffness.cpFloat, damping.cpFloat) As "?cpDampedSpringInit_pb@@YAPEAUcpDampedSpring@@PEAU1@PEAUcpBody@@1PEAUcpVect@@2NNN@Z";  = *cpDampedSpring
	cpDampedSpringNew_pb.l  (*a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect, restLength.cpFloat, stiffness.cpFloat, damping.cpFloat) As "?cpDampedSpringNew_pb@@YAPEAUcpConstraint@@PEAUcpBody@@0PEAUcpVect@@1NNN@Z";  = *cpConstraint
	cpDampedSpringSetAnchorA_pb   (*constraint.cpConstraint, *anchorA.cpVect) As "?cpDampedSpringSetAnchorA_pb@@YAXPEAUcpConstraint@@PEAUcpVect@@@Z"
	cpDampedSpringSetAnchorB_pb   (*constraint.cpConstraint, *anchorB.cpVect) As "?cpDampedSpringSetAnchorB_pb@@YAXPEAUcpConstraint@@PEAUcpVect@@@Z"
	cpDampedSpringSetDamping  (*constraint.cpConstraint, damping.cpFloat) As "cpDampedSpringSetDamping"
	cpDampedSpringSetRestLength   (*constraint.cpConstraint, restLength.cpFloat) As "cpDampedSpringSetRestLength"
	cpDampedSpringSetSpringForceFunc  (*constraint.cpConstraint, *cpDampedSpringForceFunc) As "cpDampedSpringSetSpringForceFunc"
	cpDampedSpringSetStiffness  (*constraint.cpConstraint, stiffness.cpFloat) As "cpDampedSpringSetStiffness"

	cpGearJointAlloc.l  () As "cpGearJointAlloc";  = *cpGearJoint
	cpGearJointGetPhase.cpFloat   (*constraint.cpConstraint) As "cpGearJointGetPhase"
	cpGearJointGetRatio.cpFloat   (*constraint.cpConstraint) As "cpGearJointGetRatio"
	cpGearJointInit.l   (*joint.cpGearJoint, *a.cpBody, *b.cpBody, phase.cpFloat, ratio.cpFloat) As "cpGearJointInit";  = *cpGearJoint
	cpGearJointNew.l  (*a.cpBody, *b.cpBody, phase.cpFloat, ratio.cpFloat) As "cpGearJointNew";  = *cpConstraint
	cpGearJointSetPhase   (*constraint.cpConstraint, phase.cpFloat) As "cpGearJointSetPhase"
	cpGearJointSetRatio   (*constraint.cpConstraint, ratio.cpFloat) As "cpGearJointSetRatio"

	cpGrooveJointAlloc.l  () As "cpGrooveJointAlloc";  = *cpGrooveJoint
	cpGrooveJointGetAnchorB_pb  (*constraint.cpConstraint, *lp_out.cpVect) As "?cpGrooveJointGetAnchorB_pb@@YAXPEBUcpConstraint@@PEAUcpVect@@@Z"
	cpGrooveJointGetGrooveA_pb  (*constraint.cpConstraint, *lp_out.cpVect) As "?cpGrooveJointGetGrooveA_pb@@YAXPEBUcpConstraint@@PEAUcpVect@@@Z"
	cpGrooveJointGetGrooveB_pb  (*constraint.cpConstraint, *lp_out.cpVect) As "?cpGrooveJointGetGrooveB_pb@@YAXPEBUcpConstraint@@PEAUcpVect@@@Z"
	cpGrooveJointInit_pb.l  (*joint.cpGrooveJoint, *a.cpBody, *b.cpBody, *groove_a.cpVect, *groove_b.cpVect, *anchorB.cpVect) As "?cpGrooveJointInit_pb@@YAPEAUcpGrooveJoint@@PEAU1@PEAUcpBody@@1PEAUcpVect@@22@Z";  = *cpGrooveJoint
	cpGrooveJointNew_pb.l   (*a.cpBody, *b.cpBody, *groove_a.cpVect, *groove_b.cpVect, *anchorB.cpVect) As "?cpGrooveJointNew_pb@@YAPEAUcpConstraint@@PEAUcpBody@@0PEAUcpVect@@11@Z";  = *cpConstraint
	cpGrooveJointSetAnchorB_pb  (*constraint.cpConstraint, *anchorB.cpVect) As "?cpGrooveJointSetAnchorB_pb@@YAXPEAUcpConstraint@@PEAUcpVect@@@Z"
	cpGrooveJointSetGrooveA_pb  (*constraint.cpConstraint, *value.cpVect) As "?cpGrooveJointSetGrooveA_pb@@YAXPEAUcpConstraint@@PEAUcpVect@@@Z"
	cpGrooveJointSetGrooveB_pb  (*constraint.cpConstraint, *value.cpVect) As "?cpGrooveJointSetGrooveB_pb@@YAXPEAUcpConstraint@@PEAUcpVect@@@Z"

	cpHashSetCount.l  (*set.cpHashSet) As "?cpHashSetCount@@YAHPEAUcpHashSet@@@Z"
	cpHashSetEach   (*set.cpHashSet, *cpHashSetIteratorFunc, *data) As "?cpHashSetEach@@YAXPEAUcpHashSet@@P6AXPEAX1@Z1@Z"
	cpHashSetFilter   (*set.cpHashSet, *cpHashSetFilterFunc, *data) As "?cpHashSetFilter@@YAXPEAUcpHashSet@@P6AHPEAX1@Z1@Z"
	cpHashSetFind.l   (*set.cpHashSet, hash.cpHashValue, *ptr) As "?cpHashSetFind@@YAPEAXPEAUcpHashSet@@_KPEAX@Z";  = *void
	cpHashSetFree   (*set.cpHashSet) As "?cpHashSetFree@@YAXPEAUcpHashSet@@@Z"
	cpHashSetInsert.l   (*set.cpHashSet, hash.cpHashValue, *ptr, *cpHashSetTransFunc, *data) As "?cpHashSetInsert@@YAPEAXPEAUcpHashSet@@_KPEAXP6APEAX22@Z2@Z";  = *void
	cpHashSetNew.l  (size.l, *cpHashSetEqlFunc) As "?cpHashSetNew@@YAPEAUcpHashSet@@HP6AHPEAX0@Z@Z";  = *cpHashSet
	cpHashSetRemove.l   (*set.cpHashSet, hash.cpHashValue, *ptr) As "?cpHashSetRemove@@YAPEAXPEAUcpHashSet@@_KPEAX@Z";  = *void
	cpHashSetSetDefaultValue  (*set.cpHashSet, *default_value) As "?cpHashSetSetDefaultValue@@YAXPEAUcpHashSet@@PEAX@Z"

	cpLoopIndexes   (*verts.cpVect, count.l, *start.Long, *end.Long) As "?cpLoopIndexes@@YAXPEBUcpVect@@HPEAH1@Z"
; 	cpMessage   (*condition.Ascii, *file.Ascii, line.l, isError.l, isHardError.l, *message.Ascii, ...) As "cpMessage"

	cpMomentForBox.cpFloat  (m.cpFloat, width.cpFloat, height.cpFloat) As "cpMomentForBox"
	cpMomentForBox2_pb.cpFloat  (m.cpFloat, *box.cpBB) As "?cpMomentForBox2_pb@@YANNPEAUcpBB@@@Z"
	cpMomentForCircle_pb.cpFloat  (m.cpFloat, r1.cpFloat, r2.cpFloat, *offset.cpVect) As "?cpMomentForCircle_pb@@YANNNNPEAUcpVect@@@Z"
	cpMomentForPoly_pb.cpFloat  (m.cpFloat, count.l, *verts.cpVect, *offset.cpVect, r.cpFloat) As "?cpMomentForPoly_pb@@YANNHPEBUcpVect@@PEAU1@N@Z"
	cpMomentForSegment_pb.cpFloat   (m.cpFloat, *a.cpVect, *b.cpVect, r.cpFloat) As "?cpMomentForSegment_pb@@YANNPEAUcpVect@@0N@Z"

	cpPinJointAlloc.l   () As "cpPinJointAlloc";  = *cpPinJoint
	cpPinJointGetAnchorA_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpPinJointGetAnchorA_pb@@YAXPEBUcpConstraint@@PEAUcpVect@@@Z"
	cpPinJointGetAnchorB_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpPinJointGetAnchorB_pb@@YAXPEBUcpConstraint@@PEAUcpVect@@@Z"
	cpPinJointGetDist.cpFloat   (*constraint.cpConstraint) As "cpPinJointGetDist"
	cpPinJointInit_pb.l   (*joint.cpPinJoint, *a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect) As "?cpPinJointInit_pb@@YAPEAUcpPinJoint@@PEAU1@PEAUcpBody@@1PEAUcpVect@@2@Z";  = *cpPinJoint
	cpPinJointNew_pb.l  (*a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect) As "?cpPinJointNew_pb@@YAPEAUcpConstraint@@PEAUcpBody@@0PEAUcpVect@@1@Z";  = *cpConstraint
	cpPinJointSetAnchorA_pb   (*constraint.cpConstraint, *anchorA.cpVect) As "?cpPinJointSetAnchorA_pb@@YAXPEAUcpConstraint@@PEAUcpVect@@@Z"
	cpPinJointSetAnchorB_pb   (*constraint.cpConstraint, *anchorB.cpVect) As "?cpPinJointSetAnchorB_pb@@YAXPEAUcpConstraint@@PEAUcpVect@@@Z"
	cpPinJointSetDist   (*constraint.cpConstraint, dist.cpFloat) As "cpPinJointSetDist"

	cpPivotJointAlloc.l   () As "cpPivotJointAlloc";  = *cpPivotJoint
	cpPivotJointGetAnchorA_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpPivotJointGetAnchorA_pb@@YAXPEBUcpConstraint@@PEAUcpVect@@@Z"
	cpPivotJointGetAnchorB_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpPivotJointGetAnchorB_pb@@YAXPEBUcpConstraint@@PEAUcpVect@@@Z"
	cpPivotJointInit_pb.l   (*joint.cpPivotJoint, *a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect) As "?cpPivotJointInit_pb@@YAPEAUcpPivotJoint@@PEAU1@PEAUcpBody@@1PEAUcpVect@@2@Z";  = *cpPivotJoint
	cpPivotJointNew2_pb.l   (*a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect) As "?cpPivotJointNew2_pb@@YAPEAUcpConstraint@@PEAUcpBody@@0PEAUcpVect@@1@Z";  = *cpConstraint
	cpPivotJointNew_pb.l  (*a.cpBody, *b.cpBody, *pivot.cpVect) As "?cpPivotJointNew_pb@@YAPEAUcpConstraint@@PEAUcpBody@@0PEAUcpVect@@@Z";  = *cpConstraint
	cpPivotJointSetAnchorA_pb   (*constraint.cpConstraint, *anchorA.cpVect) As "?cpPivotJointSetAnchorA_pb@@YAXPEAUcpConstraint@@PEAUcpVect@@@Z"
	cpPivotJointSetAnchorB_pb   (*constraint.cpConstraint, *anchorB.cpVect) As "?cpPivotJointSetAnchorB_pb@@YAXPEAUcpConstraint@@PEAUcpVect@@@Z"

	cpPolyShapeAlloc.l  () As "cpPolyShapeAlloc";  = *cpPolyShape
	cpPolyShapeGetCount.l   (*shape.cpShape) As "cpPolyShapeGetCount"
	cpPolyShapeGetRadius.cpFloat  (*shape.cpShape) As "cpPolyShapeGetRadius"
	cpPolyShapeGetVert_pb   (*shape.cpShape, i.l, *lp_out.cpVect) As "?cpPolyShapeGetVert_pb@@YAXPEBUcpShape@@HPEAUcpVect@@@Z"
	cpPolyShapeInit_pb.l  (*poly.cpPolyShape, *body.cpBody, count.l, *verts.cpVect, *transform.cpTransform, radius.cpFloat) As "?cpPolyShapeInit_pb@@YAPEAUcpPolyShape@@PEAU1@PEAUcpBody@@HPEBUcpVect@@PEAUcpTransform@@N@Z";  = *cpPolyShape
	cpPolyShapeInitRaw.l  (*poly.cpPolyShape, *body.cpBody, count.l, *verts.cpVect, radius.cpFloat) As "cpPolyShapeInitRaw";  = *cpPolyShape
	cpPolyShapeNew_pb.l   (*body.cpBody, count.l, *verts.cpVect, *transform.cpTransform, radius.cpFloat) As "?cpPolyShapeNew_pb@@YAPEAUcpShape@@PEAUcpBody@@HPEBUcpVect@@PEAUcpTransform@@N@Z";  = *cpShape
	cpPolyShapeNewRaw.l   (*body.cpBody, count.l, *verts.cpVect, radius.cpFloat) As "cpPolyShapeNewRaw";  = *cpShape
	cpPolyShapeSetRadius  (*shape.cpShape, radius.cpFloat) As "cpPolyShapeSetRadius"
	cpPolyShapeSetVerts_pb  (*shape.cpShape, count.l, *verts.cpVect, *transform.cpTransform) As "?cpPolyShapeSetVerts_pb@@YAXPEAUcpShape@@HPEAUcpVect@@PEAUcpTransform@@@Z"
	cpPolyShapeSetVertsRaw  (*shape.cpShape, count.l, *verts.cpVect) As "cpPolyShapeSetVertsRaw"

	cpRatchetJointAlloc.l   () As "cpRatchetJointAlloc";  = *cpRatchetJoint
	cpRatchetJointGetAngle.cpFloat  (*constraint.cpConstraint) As "cpRatchetJointGetAngle"
	cpRatchetJointGetPhase.cpFloat  (*constraint.cpConstraint) As "cpRatchetJointGetPhase"
	cpRatchetJointGetRatchet.cpFloat  (*constraint.cpConstraint) As "cpRatchetJointGetRatchet"
	cpRatchetJointInit.l  (*joint.cpRatchetJoint, *a.cpBody, *b.cpBody, phase.cpFloat, ratchet.cpFloat) As "cpRatchetJointInit";  = *cpRatchetJoint
	cpRatchetJointNew.l   (*a.cpBody, *b.cpBody, phase.cpFloat, ratchet.cpFloat) As "cpRatchetJointNew";  = *cpConstraint
	cpRatchetJointSetAngle  (*constraint.cpConstraint, angle.cpFloat) As "cpRatchetJointSetAngle"
	cpRatchetJointSetPhase  (*constraint.cpConstraint, phase.cpFloat) As "cpRatchetJointSetPhase"
	cpRatchetJointSetRatchet  (*constraint.cpConstraint, ratchet.cpFloat) As "cpRatchetJointSetRatchet"

	cpRotaryLimitJointAlloc.l   () As "cpRotaryLimitJointAlloc";  = *cpRotaryLimitJoint
	cpRotaryLimitJointGetMax.cpFloat  (*constraint.cpConstraint) As "cpRotaryLimitJointGetMax"
	cpRotaryLimitJointGetMin.cpFloat  (*constraint.cpConstraint) As "cpRotaryLimitJointGetMin"
	cpRotaryLimitJointInit.l  (*joint.cpRotaryLimitJoint, *a.cpBody, *b.cpBody, min.cpFloat, max.cpFloat) As "cpRotaryLimitJointInit";  = *cpRotaryLimitJoint
	cpRotaryLimitJointNew.l   (*a.cpBody, *b.cpBody, min.cpFloat, max.cpFloat) As "cpRotaryLimitJointNew";  = *cpConstraint
	cpRotaryLimitJointSetMax  (*constraint.cpConstraint, max.cpFloat) As "cpRotaryLimitJointSetMax"
	cpRotaryLimitJointSetMin  (*constraint.cpConstraint, min.cpFloat) As "cpRotaryLimitJointSetMin"

	cpSegmentShapeAlloc.l   () As "cpSegmentShapeAlloc";  = *cpSegmentShape
	cpSegmentShapeGetA_pb   (*shape.cpShape, *lp_out.cpVect) As "?cpSegmentShapeGetA_pb@@YAXPEBUcpShape@@PEAUcpVect@@@Z"
	cpSegmentShapeGetB_pb   (*shape.cpShape, *lp_out.cpVect) As "?cpSegmentShapeGetB_pb@@YAXPEBUcpShape@@PEAUcpVect@@@Z"
	cpSegmentShapeGetNormal_pb  (*shape.cpShape, *lp_out.cpVect) As "?cpSegmentShapeGetNormal_pb@@YAXPEBUcpShape@@PEAUcpVect@@@Z"
	cpSegmentShapeGetRadius.cpFloat   (*shape.cpShape) As "cpSegmentShapeGetRadius"
	cpSegmentShapeInit_pb.l   (*seg.cpSegmentShape, *body.cpBody, *a.cpVect, *b.cpVect, r.cpFloat) As "?cpSegmentShapeInit_pb@@YAPEAUcpSegmentShape@@PEAU1@PEAUcpBody@@PEAUcpVect@@2N@Z";  = *cpSegmentShape
	cpSegmentShapeNew_pb.l  (*body.cpBody, *a.cpVect, *b.cpVect, r.cpFloat) As "?cpSegmentShapeNew_pb@@YAPEAUcpShape@@PEAUcpBody@@PEAUcpVect@@1N@Z";  = *cpShape
	cpSegmentShapeSetEndpoints_pb   (*shape.cpShape, *a.cpVect, *b.cpVect) As "?cpSegmentShapeSetEndpoints_pb@@YAXPEAUcpShape@@PEAUcpVect@@1@Z"
	cpSegmentShapeSetNeighbors_pb   (*shape.cpShape, *prev.cpVect, *next.cpVect) As "?cpSegmentShapeSetNeighbors_pb@@YAXPEAUcpShape@@PEAUcpVect@@1@Z"
	cpSegmentShapeSetRadius   (*shape.cpShape, radius.cpFloat) As "cpSegmentShapeSetRadius"

	cpShapeCacheBB_pb   (*shape.cpShape, *lp_out.cpBB) As "?cpShapeCacheBB_pb@@YAXPEAUcpShape@@PEAUcpBB@@@Z"
	cpShapeDestroy  (*shape.cpShape) As "cpShapeDestroy"
	cpShapeFree   (*shape.cpShape) As "cpShapeFree"
	cpShapeGetArea.cpFloat  (*shape.cpShape) As "cpShapeGetArea"
	cpShapeGetBB_pb   (*shape.cpShape, *lp_out.cpBB) As "?cpShapeGetBB_pb@@YAXPEBUcpShape@@PEAUcpBB@@@Z"
	cpShapeGetBody.l  (*shape.cpShape) As "cpShapeGetBody";  = *cpBody
	cpShapeGetCenterOfGravity_pb  (*shape.cpShape, *lp_out.cpVect) As "?cpShapeGetCenterOfGravity_pb@@YAXPEAUcpShape@@PEAUcpVect@@@Z"
	cpShapeGetCollisionType.cpCollisionType   (*shape.cpShape) As "cpShapeGetCollisionType"
	cpShapeGetDensity.cpFloat   (*shape.cpShape) As "cpShapeGetDensity"
	cpShapeGetElasticity.cpFloat  (*shape.cpShape) As "cpShapeGetElasticity"
	cpShapeGetFilter_pb   (*shape.cpShape, *lp_out.cpShapeFilter) As "?cpShapeGetFilter_pb@@YAXPEBUcpShape@@PEAUcpShapeFilter@@@Z"
	cpShapeGetFriction.cpFloat  (*shape.cpShape) As "cpShapeGetFriction"
	cpShapeGetMass.cpFloat  (*shape.cpShape) As "cpShapeGetMass"
	cpShapeGetMoment.cpFloat  (*shape.cpShape) As "cpShapeGetMoment"
	cpShapeGetSensor.cpBool   (*shape.cpShape) As "cpShapeGetSensor"
	cpShapeGetSpace.l   (*shape.cpShape) As "cpShapeGetSpace";  = *cpSpace
	cpShapeGetSurfaceVelocity_pb  (*shape.cpShape, *lp_out.cpVect) As "?cpShapeGetSurfaceVelocity_pb@@YAXPEBUcpShape@@PEAUcpVect@@@Z"
	cpShapeGetUserData.l  (*shape.cpShape) As "cpShapeGetUserData";  = *cpDataPointer
	cpShapeInit_pb.l  (*shape.cpShape, *klass.cpShapeClass, *body.cpBody, *massInfo.cpShapeMassInfo) As "?cpShapeInit_pb@@YAPEAUcpShape@@PEAU1@PEBUcpShapeClass@@PEAUcpBody@@PEAUcpShapeMassInfo@@@Z";  = *cpShape
	cpShapePointQuery_pb.cpFloat  (*shape.cpShape, *p.cpVect, *info.cpPointQueryInfo) As "?cpShapePointQuery_pb@@YANPEBUcpShape@@PEAUcpVect@@PEAUcpPointQueryInfo@@@Z"
	cpShapesCollide_pb  (*a.cpShape, *b.cpShape, *lp_out.cpContactPointSet) As "?cpShapesCollide_pb@@YAXPEBUcpShape@@0PEAUcpContactPointSet@@@Z"
	cpShapeSegmentQuery_pb.cpBool   (*shape.cpShape, *a.cpVect, *b.cpVect, radius.cpFloat, *info.cpSegmentQueryInfo) As "?cpShapeSegmentQuery_pb@@YAHPEBUcpShape@@PEAUcpVect@@1NPEAUcpSegmentQueryInfo@@@Z"
	cpShapeSetBody  (*shape.cpShape, *body.cpBody) As "cpShapeSetBody"
	cpShapeSetCollisionType   (*shape.cpShape, collisionType.cpCollisionType) As "cpShapeSetCollisionType"
	cpShapeSetDensity   (*shape.cpShape, density.cpFloat) As "cpShapeSetDensity"
	cpShapeSetElasticity  (*shape.cpShape, elasticity.cpFloat) As "cpShapeSetElasticity"
	cpShapeSetFilter_pb   (*shape.cpShape, *filter.cpShapeFilter) As "?cpShapeSetFilter_pb@@YAXPEAUcpShape@@PEAUcpShapeFilter@@@Z"
	cpShapeSetFriction  (*shape.cpShape, friction.cpFloat) As "cpShapeSetFriction"
	cpShapeSetMass  (*shape.cpShape, mass.cpFloat) As "cpShapeSetMass"
	cpShapeSetSensor  (*shape.cpShape, sensor.cpBool) As "cpShapeSetSensor"
	cpShapeSetSurfaceVelocity_pb  (*shape.cpShape, *surfaceVelocity.cpVect) As "?cpShapeSetSurfaceVelocity_pb@@YAXPEAUcpShape@@PEAUcpVect@@@Z"
	cpShapeSetUserData  (*shape.cpShape, *cpDataPointer) As "cpShapeSetUserData"
	cpShapeUpdate_pb  (*shape.cpShape, *transform.cpTransform, *lp_out.cpBB) As "?cpShapeUpdate_pb@@YAXPEAUcpShape@@PEAUcpTransform@@PEAUcpBB@@@Z"
	cpShapeUpdateFunc   (*shape.cpShape, *unused) As "?cpShapeUpdateFunc@@YAXPEAUcpShape@@PEAX@Z"

	cpSimpleMotorAlloc.l  () As "cpSimpleMotorAlloc";  = *cpSimpleMotor
	cpSimpleMotorGetRate.cpFloat  (*constraint.cpConstraint) As "cpSimpleMotorGetRate"
	cpSimpleMotorInit.l   (*joint.cpSimpleMotor, *a.cpBody, *b.cpBody, rate.cpFloat) As "cpSimpleMotorInit";  = *cpSimpleMotor
	cpSimpleMotorNew.l  (*a.cpBody, *b.cpBody, rate.cpFloat) As "cpSimpleMotorNew";  = *cpConstraint
	cpSimpleMotorSetRate  (*constraint.cpConstraint, rate.cpFloat) As "cpSimpleMotorSetRate"

	cpSlideJointAlloc.l   () As "cpSlideJointAlloc";  = *cpSlideJoint
	cpSlideJointGetAnchorA_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpSlideJointGetAnchorA_pb@@YAXPEBUcpConstraint@@PEAUcpVect@@@Z"
	cpSlideJointGetAnchorB_pb   (*constraint.cpConstraint, *lp_out.cpVect) As "?cpSlideJointGetAnchorB_pb@@YAXPEBUcpConstraint@@PEAUcpVect@@@Z"
	cpSlideJointGetMax.cpFloat  (*constraint.cpConstraint) As "cpSlideJointGetMax"
	cpSlideJointGetMin.cpFloat  (*constraint.cpConstraint) As "cpSlideJointGetMin"
	cpSlideJointInit_pb.l   (*joint.cpSlideJoint, *a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect, min.cpFloat, max.cpFloat) As "?cpSlideJointInit_pb@@YAPEAUcpSlideJoint@@PEAU1@PEAUcpBody@@1PEAUcpVect@@2NN@Z";  = *cpSlideJoint
	cpSlideJointNew_pb.l  (*a.cpBody, *b.cpBody, *anchorA.cpVect, *anchorB.cpVect, min.cpFloat, max.cpFloat) As "?cpSlideJointNew_pb@@YAPEAUcpConstraint@@PEAUcpBody@@0PEAUcpVect@@1NN@Z";  = *cpConstraint
	cpSlideJointSetAnchorA_pb   (*constraint.cpConstraint, *anchorA.cpVect) As "?cpSlideJointSetAnchorA_pb@@YAXPEAUcpConstraint@@PEAUcpVect@@@Z"
	cpSlideJointSetAnchorB_pb   (*constraint.cpConstraint, *anchorB.cpVect) As "?cpSlideJointSetAnchorB_pb@@YAXPEAUcpConstraint@@PEAUcpVect@@@Z"
	cpSlideJointSetMax  (*constraint.cpConstraint, max.cpFloat) As "cpSlideJointSetMax"
	cpSlideJointSetMin  (*constraint.cpConstraint, min.cpFloat) As "cpSlideJointSetMin"

	cpSpaceActivateBody   (*space.cpSpace, *body.cpBody) As "?cpSpaceActivateBody@@YAXPEAUcpSpace@@PEAUcpBody@@@Z"
	cpSpaceAddBody.l  (*space.cpSpace, *body.cpBody) As "cpSpaceAddBody";  = *cpBody
	cpSpaceAddCollisionHandler.l  (*space.cpSpace, a.cpCollisionType, b.cpCollisionType) As "cpSpaceAddCollisionHandler";  = *cpCollisionHandler
	cpSpaceAddConstraint.l  (*space.cpSpace, *constraint.cpConstraint) As "cpSpaceAddConstraint";  = *cpConstraint
	cpSpaceAddDefaultCollisionHandler.l   (*space.cpSpace) As "cpSpaceAddDefaultCollisionHandler";  = *cpCollisionHandler
	cpSpaceAddPostStepCallback.cpBool   (*space.cpSpace, *cpPostStepFunc, *key, *data) As "cpSpaceAddPostStepCallback"
	cpSpaceAddShape.l   (*space.cpSpace, *shape.cpShape) As "cpSpaceAddShape";  = *cpShape
	cpSpaceAddWildcardHandler.l   (*space.cpSpace, type.cpCollisionType) As "cpSpaceAddWildcardHandler";  = *cpCollisionHandler
	cpSpaceAlloc.l  () As "cpSpaceAlloc";  = *cpSpace
	cpSpaceArbiterSetFilter.cpBool  (*arb.cpArbiter, *space.cpSpace) As "?cpSpaceArbiterSetFilter@@YAHPEAUcpArbiter@@PEAUcpSpace@@@Z"
	cpSpaceBBQuery_pb   (*space.cpSpace, *bb.cpBB, *filter.cpShapeFilter, *cpSpaceBBQueryFunc, *data) As "?cpSpaceBBQuery_pb@@YAXPEAUcpSpace@@PEAUcpBB@@PEAUcpShapeFilter@@P6AXPEAUcpShape@@PEAX@Z4@Z"
	cpSpaceCollideShapes.cpCollisionID  (*a.cpShape, *b.cpShape, id.cpCollisionID, *space.cpSpace) As "?cpSpaceCollideShapes@@YAIPEAUcpShape@@0IPEAUcpSpace@@@Z"
	cpSpaceContainsBody.cpBool  (*space.cpSpace, *body.cpBody) As "cpSpaceContainsBody"
	cpSpaceContainsConstraint.cpBool  (*space.cpSpace, *constraint.cpConstraint) As "cpSpaceContainsConstraint"
	cpSpaceContainsShape.cpBool   (*space.cpSpace, *shape.cpShape) As "cpSpaceContainsShape"
; 	cpSpaceDebugDraw  (*space.cpSpace, *options.cpSpaceDebugDrawOptions) As "cpSpaceDebugDraw"
	cpSpaceDestroy  (*space.cpSpace) As "cpSpaceDestroy"
	cpSpaceEachBody   (*space.cpSpace, *cpSpaceBodyIteratorFunc, *data) As "cpSpaceEachBody"
	cpSpaceEachConstraint   (*space.cpSpace, *cpSpaceConstraintIteratorFunc, *data) As "cpSpaceEachConstraint"
	cpSpaceEachShape  (*space.cpSpace, *cpSpaceShapeIteratorFunc, *data) As "cpSpaceEachShape"
	cpSpaceFilterArbiters   (*space.cpSpace, *body.cpBody, *filter.cpShape) As "?cpSpaceFilterArbiters@@YAXPEAUcpSpace@@PEAUcpBody@@PEAUcpShape@@@Z"
	cpSpaceFree   (*space.cpSpace) As "cpSpaceFree"
	cpSpaceGetCollisionBias.cpFloat   (*space.cpSpace) As "cpSpaceGetCollisionBias"
	cpSpaceGetCollisionPersistence.cpTimestamp  (*space.cpSpace) As "cpSpaceGetCollisionPersistence"
	cpSpaceGetCollisionSlop.cpFloat   (*space.cpSpace) As "cpSpaceGetCollisionSlop"
	cpSpaceGetCurrentTimeStep.cpFloat   (*space.cpSpace) As "cpSpaceGetCurrentTimeStep"
	cpSpaceGetDamping.cpFloat   (*space.cpSpace) As "cpSpaceGetDamping"
	cpSpaceGetGravity_pb  (*space.cpSpace, *lp_out.cpVect) As "?cpSpaceGetGravity_pb@@YAXPEBUcpSpace@@PEAUcpVect@@@Z"
	cpSpaceGetIdleSpeedThreshold.cpFloat  (*space.cpSpace) As "cpSpaceGetIdleSpeedThreshold"
	cpSpaceGetIterations.l  (*space.cpSpace) As "cpSpaceGetIterations"
	cpSpaceGetPostStepCallback.l  (*space.cpSpace, *key) As "?cpSpaceGetPostStepCallback@@YAPEAUcpPostStepCallback@@PEAUcpSpace@@PEAX@Z";  = *cpPostStepCallback
	cpSpaceGetSleepTimeThreshold.cpFloat  (*space.cpSpace) As "cpSpaceGetSleepTimeThreshold"
	cpSpaceGetStaticBody.l  (*space.cpSpace) As "cpSpaceGetStaticBody";  = *cpBody
	cpSpaceGetUserData.l  (*space.cpSpace) As "cpSpaceGetUserData";  = *cpDataPointer
	cpSpaceHashAlloc.l  () As "cpSpaceHashAlloc";  = *cpSpaceHash
	cpSpaceHashInit.l   (*hash.cpSpaceHash, celldim.cpFloat, numcells.l, *cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "cpSpaceHashInit";  = *cpSpatialIndex
	cpSpaceHashNew.l  (celldim.cpFloat, cells.l, *cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "cpSpaceHashNew";  = *cpSpatialIndex
	cpSpaceHashResize   (*hash.cpSpaceHash, celldim.cpFloat, numcells.l) As "cpSpaceHashResize"
	cpSpaceInit.l   (*space.cpSpace) As "cpSpaceInit";  = *cpSpace
	cpSpaceIsLocked.cpBool  (*space.cpSpace) As "cpSpaceIsLocked"
	cpSpaceLock   (*space.cpSpace) As "?cpSpaceLock@@YAXPEAUcpSpace@@@Z"
	cpSpaceNew.l  () As "cpSpaceNew";  = *cpSpace
	cpSpacePointQuery_pb  (*space.cpSpace, *point.cpVect, maxDistance.cpFloat, *filter.cpShapeFilter, *cpSpacePointQueryFunc, *data) As "?cpSpacePointQuery_pb@@YAXPEAUcpSpace@@PEAUcpVect@@NPEAUcpShapeFilter@@P6AXPEAUcpShape@@1N1PEAX@Z4@Z"
	cpSpacePointQueryNearest_pb.l   (*space.cpSpace, *point.cpVect, maxDistance.cpFloat, *filter.cpShapeFilter, *out.cpPointQueryInfo) As "?cpSpacePointQueryNearest_pb@@YAPEAUcpShape@@PEAUcpSpace@@PEAUcpVect@@NPEAUcpShapeFilter@@PEAUcpPointQueryInfo@@@Z";  = *cpShape
	cpSpaceProcessComponents  (*space.cpSpace, dt.cpFloat) As "?cpSpaceProcessComponents@@YAXPEAUcpSpace@@N@Z"
	cpSpacePushContacts   (*space.cpSpace, count.l) As "?cpSpacePushContacts@@YAXPEAUcpSpace@@H@Z"
	cpSpacePushFreshContactBuffer   (*space.cpSpace) As "?cpSpacePushFreshContactBuffer@@YAXPEAUcpSpace@@@Z"
	cpSpaceReindexShape   (*space.cpSpace, *shape.cpShape) As "cpSpaceReindexShape"
	cpSpaceReindexShapesForBody   (*space.cpSpace, *body.cpBody) As "cpSpaceReindexShapesForBody"
	cpSpaceReindexStatic  (*space.cpSpace) As "cpSpaceReindexStatic"
	cpSpaceRemoveBody   (*space.cpSpace, *body.cpBody) As "cpSpaceRemoveBody"
	cpSpaceRemoveConstraint   (*space.cpSpace, *constraint.cpConstraint) As "cpSpaceRemoveConstraint"
	cpSpaceRemoveShape  (*space.cpSpace, *shape.cpShape) As "cpSpaceRemoveShape"
	cpSpaceSegmentQuery_pb  (*space.cpSpace, *start.cpVect, *end.cpVect, radius.cpFloat, *filter.cpShapeFilter, *cpSpaceSegmentQueryFunc, *data) As "?cpSpaceSegmentQuery_pb@@YAXPEAUcpSpace@@PEAUcpVect@@1NPEAUcpShapeFilter@@P6AXPEAUcpShape@@11NPEAX@Z4@Z"
	cpSpaceSegmentQueryFirst_pb.l   (*space.cpSpace, *start.cpVect, *end.cpVect, radius.cpFloat, *filter.cpShapeFilter, *out.cpSegmentQueryInfo) As "?cpSpaceSegmentQueryFirst_pb@@YAPEAUcpShape@@PEAUcpSpace@@PEAUcpVect@@1NPEAUcpShapeFilter@@PEAUcpSegmentQueryInfo@@@Z";  = *cpShape
	cpSpaceSetCollisionBias   (*space.cpSpace, collisionBias.cpFloat) As "cpSpaceSetCollisionBias"
	cpSpaceSetCollisionPersistence  (*space.cpSpace, collisionPersistence.cpTimestamp) As "cpSpaceSetCollisionPersistence"
	cpSpaceSetCollisionSlop   (*space.cpSpace, collisionSlop.cpFloat) As "cpSpaceSetCollisionSlop"
	cpSpaceSetDamping   (*space.cpSpace, damping.cpFloat) As "cpSpaceSetDamping"
	cpSpaceSetGravity_pb  (*space.cpSpace, *gravity.cpVect) As "?cpSpaceSetGravity_pb@@YAXPEAUcpSpace@@PEAUcpVect@@@Z"
	cpSpaceSetIdleSpeedThreshold  (*space.cpSpace, idleSpeedThreshold.cpFloat) As "cpSpaceSetIdleSpeedThreshold"
	cpSpaceSetIterations  (*space.cpSpace, iterations.l) As "cpSpaceSetIterations"
	cpSpaceSetSleepTimeThreshold  (*space.cpSpace, sleepTimeThreshold.cpFloat) As "cpSpaceSetSleepTimeThreshold"
	cpSpaceSetStaticBody  (*space.cpSpace, *body.cpBody) As "?cpSpaceSetStaticBody@@YAXPEAUcpSpace@@PEAUcpBody@@@Z"
	cpSpaceSetUserData  (*space.cpSpace, *cpDataPointer) As "cpSpaceSetUserData"
	cpSpaceShapeQuery.cpBool  (*space.cpSpace, *shape.cpShape, *cpSpaceShapeQueryFunc, *data) As "cpSpaceShapeQuery"
	cpSpaceStep   (*space.cpSpace, dt.cpFloat) As "cpSpaceStep"
	cpSpaceUnlock   (*space.cpSpace, runPostStep.cpBool) As "?cpSpaceUnlock@@YAXPEAUcpSpace@@H@Z"
	cpSpaceUseSpatialHash   (*space.cpSpace, cDim.cpFloat, count.l) As "cpSpaceUseSpatialHash"

	cpSpatialIndexCollideStatic   (*dynamicIndex.cpSpatialIndex, *staticIndex.cpSpatialIndex, *cpSpatialIndexQueryFunc, *data) As "cpSpatialIndexCollideStatic"
	cpSpatialIndexFree  (*index.cpSpatialIndex) As "cpSpatialIndexFree"
	cpSpatialIndexInit.l  (*index.cpSpatialIndex, *klass.cpSpatialIndexClass, *cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "?cpSpatialIndexInit@@YAPEAUcpSpatialIndex@@PEAU1@PEAUcpSpatialIndexClass@@P6A?AUcpBB@@PEAX@Z0@Z";  = *cpSpatialIndex

	cpSweep1DAlloc.l  () As "cpSweep1DAlloc";  = *cpSweep1D
	cpSweep1DInit.l   (*sweep.cpSweep1D, *cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "cpSweep1DInit";  = *cpSpatialIndex
	cpSweep1DNew.l  (*cpSpatialIndexBBFunc, *staticIndex.cpSpatialIndex) As "cpSweep1DNew";  = *cpSpatialIndex
EndImport
CompilerEndIf

; ========================================================================================================
;	Chipmunk constants
; ========================================================================================================

	Global cpvzero.cpVect ; PB doesn't allow use structure as constant, so just declare it as global var

	Global cpvzerofilter.cpShapeFilter  ; this filter will include any shape (all 32 bits set up)
		cpvzerofilter\cat_belongs_to 	= %11111111111111111111111111111111
		cpvzerofilter\cat_collide_with 	= %11111111111111111111111111111111
	
	#CP_NO_GROUP = 0 ; value For cpShape.group signifying that a shape is in no group.
	
	#CP_FALSE = 0	 ; boolean values
	#CP_TRUE = 1

	#CPFLOAT_MIN = 2.2250738585072014e-308 ; /* min positive value */

; ========================================================================================================
;	Useful routines / typedefs
; ========================================================================================================
	;	Callback prototypes. Should use their syntax to make own callbacks implementations
	; ====================================================================================================
	
	; TODO: those 2 still not working, should do something with their return values
	; ProcedureC.cpBB cpSpatialIndexBBFunc (*obj);; RETURN:  // Spatial index bounding box callback function type. The spatial index calls this function and passes you a pointer to an object you added when it needs to get the bounding box associated with that object.
	; ProcedureC.cpVect cpBBTreeVelocityFunc (*obj); RETURN:  // Bounding box tree velocity callback function. This function should return an estimate for the object's velocity.

	; ProcedureC cpBodyVelocityFunc (*body.cpBody, *gravity.cpVect, damping.cpFloat, dt.cpFloat) ; Rigid body velocity update function type.
	; ProcedureC cpBodyPositionFunc (*body.cpBody, dt.cpFloat) ; Rigid body position update function type.
	; ProcedureC cpBodyShapeIteratorFunc (*body.cpBody, *shape.cpShape, *data); Body/shape iterator callback function type. 
	; ProcedureC cpBodyConstraintIteratorFunc (*body.cpBody, *constraint.cpConstraint, *data); Body/constraint iterator callback function type. 
	; ProcedureC cpBodyArbiterIteratorFunc (*body.cpBody, *arbiter.cpArbiter, *data); Body/arbiter iterator callback function type. 
	
	; ProcedureC cpConstraintPreSolveFunc (*constraint.cpConstraint, *space.cpSpace); Callback function type that gets called before solving a joint.
	; ProcedureC cpConstraintPostSolveFunc (*constraint.cpConstraint, *space.cpSpace); Callback function type that gets called after solving a joint.
	
	; ProcedureC.cpFloat cpDampedSpringForceFunc (*spring.cpConstraint, dist.cpFloat); Function type used for damped spring force callbacks.
	; ProcedureC.cpFloat cpDampedRotarySpringTorqueFunc (*spring.cpConstraint, relativeAngle.cpFloat); Function type used for damped rotary spring force callbacks.
	
	; ProcedureC.cpBool cpHashSetEqlFunc (*ptr, *elt)
	; ProcedureC.i cpHashSetTransFunc (*ptr, *data) ; RETURN: *Pointer
	; ProcedureC cpHashSetIteratorFunc (*elt, *data)
	; ProcedureC.cpBool cpHashSetFilterFunc (*elt, *data)
	
	; ProcedureC cpSpaceBodyIteratorFunc (*body.cpBody, *data); Space/body iterator callback function type.
	; ProcedureC cpSpaceShapeIteratorFunc (*shape.cpShape, *data); Space/body iterator callback function type.
	; ProcedureC cpSpaceConstraintIteratorFunc (*constraint.cpConstraint, *data); Space/constraint iterator callback function type.
	; ProcedureC cpSpacePointQueryFunc (*shape.cpShape, *point.cpVect, distance.cpFloat, *gradient.cpVect, *data); Nearest point query callback function type.
	; ProcedureC cpSpaceSegmentQueryFunc (*shape.cpShape, *point.cpVect, *normal.cpVect, alpha.cpFloat, *data); Segment query callback function type.
	; ProcedureC cpSpaceBBQueryFunc (*shape.cpShape, *data); Rectangle Query callback function type.
	; ProcedureC cpSpaceShapeQueryFunc (*shape.cpShape, *points.cpContactPointSet, *data); Shape query callback function type.
	
	; ProcedureC cpPostStepFunc (*space.cpSpace, *key, *data); Post Step callback function type.
	
	; ProcedureC.cpCollisionID cpSpatialIndexQueryFunc (*obj1, *obj2, id.cpCollisionID, *data); Spatial query callback function type.
	
	; ====================================================================================================
	;	cpv_ functions   /   other math routines ported from Chipmunk
	; ====================================================================================================
	Procedure cpvadd(*lp_out.cpVect, *v1.cpVect, *v2.cpVect);Add two vectors
		*lp_out\X = *v1\x + *v2\x
		*lp_out\Y = *v1\y + *v2\y
	EndProcedure
	
	Procedure cpvsub(*lp_out.cpVect, *v1.cpVect, *v2.cpVect);Subtract two vectors.
		*lp_out\X = *v1\x - *v2\x
		*lp_out\Y = *v1\y - *v2\y
	EndProcedure

	Procedure cpvneg(*lp_out.cpVect, *v.cpVect) ;Negate a vector.
		*lp_out\X = -*v\X
		*lp_out\Y = -*v\Y
	EndProcedure
	
	Procedure cpvmult(*lp_out.cpVect, *v.cpVect, s.cpFloat);Scalar multiplication.
		*lp_out\X = *v\x*s
		*lp_out\Y = *v\y*s
	EndProcedure

	Procedure.cpFloat cpvdot(*v1.cpVect, *v2.cpVect);Vector dot product.
		ProcedureReturn *v1\x * *v2\x + *v1\y * *v2\y
	EndProcedure
	
	Procedure.cpFloat cpvcross(*v1.cpVect, *v2.cpVect);2D vector cross product analog. The cross product of 2D vectors results in a 3D vector With only a z component. This function returns the magnitude of the z value.
		ProcedureReturn *v1\x * *v2\y - *v1\y * *v2\x
	EndProcedure

	Procedure cpvperp(*lp_out.cpVect, *v.cpVect);Returns a perpendicular vector. (90 degree rotation)
		*lp_out\X = -*v\y
		*lp_out\Y = *v\x
	EndProcedure
	
	Procedure cpvrperp(*lp_out.cpVect, *v.cpVect);Returns a perpendicular vector. (-90 degree rotation)
		*lp_out\X = *v\y
		*lp_out\Y = -*v\x
	EndProcedure
	
	Procedure cpvproject(*lp_out.cpVect, *v1.cpVect, *v2.cpVect);Returns the vector projection of v1 onto v2.
		cpvmult (*lp_out, *v2, cpvdot(*v1, *v2) / cpvdot(*v2, *v2))
	EndProcedure
	
	Procedure cpvforangle(*lp_out.cpVect, a.cpFloat);Returns the unit length vector For the given angle (in radians).
		*lp_out\X = Cos(a)
		*lp_out\Y = Sin(a)
	EndProcedure
	
	Procedure.cpFloat cpvtoangle(*v.cpVect);Returns the angular direction v is pointing in (in radians).
		ProcedureReturn ATan2(*v\y, *v\x)
	EndProcedure
	
	Procedure cpvrotate(*lp_out.cpVect, *v1.cpVect, *v2.cpVect);Uses complex number multiplication To rotate v1 by v2. Scaling will occur If v1 is Not a unit vector.
		*lp_out\X = *v1\x * *v2\x - *v1\y * *v2\y
		*lp_out\Y = *v1\x * *v2\y + *v1\y * *v2\x
	EndProcedure
	
	Procedure cpvunrotate(*lp_out.cpVect, *v1.cpVect, *v2.cpVect);Inverse of cpvrotate().
		*lp_out\X = *v1\x * *v2\x + *v1\y * *v2\y
		*lp_out\Y = *v1\y * *v2\x - *v1\x * *v2\y
	EndProcedure
	
	Procedure.cpFloat cpvlengthsq(*v.cpVect);Returns the squared length of v. Faster than cpvlength() when you only need To compare lengths.
		ProcedureReturn cpvdot(*v, *v)
	EndProcedure
	
	Procedure.cpFloat cpvlength(*v.cpVect);Returns the length of v.
		ProcedureReturn Sqr(cpvdot(*v, *v))
	EndProcedure

	Procedure cpvlerp(*lp_out.cpVect, *v1.cpVect, *v2.cpVect, t.cpFloat);Linearly interpolate between v1 And v2.
		Define LPOut2.cpVect, LPOut3.cpVect
			cpvmult(LPOut2, *v1, 1.0 - t)
			cpvmult(LPOut3, *v2, t)
		
		cpvadd(*lp_out, LPOut2, LPOut3)
	EndProcedure
	
	Procedure cpvnormalize(*lp_out.cpVect, *v.cpVect);Returns a normalized copy of v.
		;Neat trick I saw somewhere to avoid div/0
		cpvmult(*lp_out, *v, 1.0 / (cpvlength(*v) + #CPFLOAT_MIN))
	EndProcedure

	Procedure.cpFloat cpvdist(*v1.cpVect, *v2.cpVect);Returns the distance between v1 And v2.
		Define LPOut2.cpVect
			cpvsub(LPOut2, *v1, *v2)
		ProcedureReturn cpvlength(LPOut2)
	EndProcedure
	
	Procedure.cpFloat cpvdistsq(*v1.cpVect, *v2.cpVect);Returns the squared distance between v1 And v2. Faster than cpvdist() when you only need To compare distances.
		Define LPOut2.cpVect
			cpvsub(LPOut2, *v1, *v2)
		ProcedureReturn cpvlengthsq(LPOut2)
	EndProcedure
	
	Procedure.cpBool cpvnear(*v1.cpVect, *v2.cpVect, dist.cpFloat);Returns true If the distance between v1 And v2 is less than dist.
		If cpvdistsq(*v1, *v2) < dist * dist
			ProcedureReturn #CP_TRUE
		EndIf
	EndProcedure

	Procedure.cpFloat cpfmax(a.cpFloat, b.cpFloat) ;Return the max of two cpFloats.
		If a > b
			ProcedureReturn a
		Else
			ProcedureReturn b
		EndIf
	EndProcedure
	
	Procedure.cpFloat cpfmin(a.cpFloat, b.cpFloat);Return the min of two cpFloats.
		If a < b
			ProcedureReturn a
		Else
			ProcedureReturn b
		EndIf
	EndProcedure
	
	Procedure.cpFloat cpfclamp(f.cpFloat, min.cpFloat, max.cpFloat);Clamp @c f To be between @c min And @c max.
		ProcedureReturn cpfmin(cpfmax(f, min), max)
	EndProcedure
	
	Procedure.cpFloat cpfabs(f.cpFloat);Return the absolute value of a cpFloat.
		If f < 0
			ProcedureReturn -f
		Else
			ProcedureReturn f
		EndIf
	EndProcedure

	Procedure.cpFloat cpfclamp01(f.cpFloat);Clamp @c f To be between 0 And 1.
		ProcedureReturn cpfmax(0.0, cpfmin(f, 1.0))
	EndProcedure
	
	Procedure.cpFloat cpflerp(f1.cpFloat, f2.cpFloat, t.cpFloat);Linearly interpolate (Or extrapolate) between @c f1 And @c f2 by @c t percent.
		ProcedureReturn f1 * (1.0 - t) + f2 * t
	EndProcedure
	
	Procedure.cpFloat cpflerpconst(f1.cpFloat, f2.cpFloat, d.cpFloat);Linearly interpolate from @c f1 To @c f2 by no more than @c d.
		ProcedureReturn f1 + cpfclamp(f2 - f1, -d, d)
	EndProcedure

	Procedure cpvslerp(*lp_out.cpVect, *v1.cpVect, *v2.cpVect, t.cpFloat);Spherical linearly interpolate between v1 And v2.
		Define LPOut2.cpVect, LPOut3.cpVect
			cpvnormalize(LPOut2, *v1)
			cpvnormalize(LPOut3, *v2)
		
		Define dot.cpFloat = cpvdot(LPOut2, LPOut3)
		Define omega.cpFloat = ACos(cpfclamp(dot, -1.0, 1.0))
		
		If (omega < 1e-3)
			; if the angle between two vectors is very small, lerp instead To avoid precision issues.
			cpvlerp(*lp_out, *v1, *v2, t)
		Else
			Define denom.cpFloat  = 1.0 / Sin(omega)
				cpvmult(LPOut2, *v1, Sin((1.0 - t) *omega) *denom)
				cpvmult(LPOut3, *v2, Sin(t*omega) *denom)
			
			cpvadd(*lp_out, LPOut2, LPOut3)
		EndIf
	EndProcedure
	
	Procedure cpvslerpconst(*lp_out.cpVect, *v1.cpVect, *v2.cpVect, a.cpFloat);Spherical linearly interpolate between v1 towards v2 by no more than angle a radians
		Define LPOut2.cpVect, LPOut3.cpVect
			cpvnormalize(LPOut2, *v1)
			cpvnormalize(LPOut3, *v2)
		Define dot.cpFloat = cpvdot(LPOut2, LPOut3)
		Define omega.cpFloat = ACos(cpfclamp(dot, -1.0, 1.0))
		
		cpvslerp(*lp_out, *v1, *v2, cpfmin(a, omega) / omega)
	EndProcedure
	
	Procedure cpvclamp(*lp_out.cpVect, *v.cpVect, len.cpFloat);Clamp v To length len.
		If cpvdot(*v,*v) > len * len
			Define LPOut2.cpVect
				cpvnormalize(LPOut2, *v)
	
			cpvmult(*lp_out, LPOut2, len)
		Else
			*lp_out = *v
		EndIf
	EndProcedure
	
	Procedure cpvlerpconst(*lp_out.cpVect, *v1.cpVect, *v2.cpVect, d.cpFloat);Linearly interpolate between v1 towards v2 by distance d.
		Define LPOut2.cpVect, LPOut3.cpVect
			cpvsub(LPOut3, *v2, *v1)
			cpvclamp(LPOut2, LPOut3, d)
		
		cpvadd(*lp_out, *v1, LPOut2)
	EndProcedure
	
	Procedure cpTransformTranslate(*cpTransform.cpTransform, *translate.cpVect) ; Create a transation matrix.
		With *cpTransform
			\a = 1.0
			\b = 0.0
			\c = *translate\x
			\d = 0.0
			\tx = 1.0
			\ty = *translate\y
		EndWith
	EndProcedure
	
	; ====================================================================================================
	;	Other types of functions ported from Chipmunk
	; ====================================================================================================
	Procedure cpv(*lp_out.cpVect, x.cpFloat, y.cpFloat) ; Convenience constructor For cpVect structs.
		*lp_out\X = X
		*lp_out\Y = Y
	EndProcedure
	
	Procedure cpBBNew(*lp_out.cpBB, l.cpFloat, b.cpFloat, r.cpFloat, t.cpFloat) ; Convenience constructor For cpBB structs.
		*lp_out\L = l
		*lp_out\B = b
		*lp_out\R = r
		*lp_out\T = t
	EndProcedure
	
	; ====================================================================================================
	;	Other useful functions
	; ====================================================================================================

	ProcedureC ConstraintFreeWrap(*space.cpSpace, *constraint.cpConstraint, *unused) ; ChipmunkFreeSpaceChildren
		cpSpaceRemoveConstraint(*space, *constraint)
		cpConstraintFree(*constraint)
	EndProcedure
	ProcedureC ShapeFreeWrap(*space.cpSpace, *shape.cpShape, *unused) ; ChipmunkFreeSpaceChildren
		cpSpaceRemoveShape(*space, *shape)
		cpShapeFree(*shape)
	EndProcedure
	ProcedureC BodyFreeWrap(*space.cpSpace, *body.cpBody, *unused) ; ChipmunkFreeSpaceChildren
		cpSpaceRemoveBody(*space, *body)
		cpBodyFree(*body)
	EndProcedure
	
	ProcedureC PostConstraintFree(*constraint.cpConstraint, *space.cpSpace) ; ChipmunkFreeSpaceChildren
		cpSpaceAddPostStepCallback(*space, @ConstraintFreeWrap(), *constraint, #Null)
	EndProcedure
	ProcedureC PostShapeFree(*shape.cpShape, *space.cpSpace)
		cpSpaceAddPostStepCallback(*space, @ShapeFreeWrap(), *shape, #Null) ; ChipmunkFreeSpaceChildren
	EndProcedure
	ProcedureC PostBodyFree(*body.cpBody, *space.cpSpace)
		cpSpaceAddPostStepCallback(*space, @BodyFreeWrap(), *body, #Null) ; ChipmunkFreeSpaceChildren
	EndProcedure
	
	ProcedureC cpPostBodyShapesFree (*body.cpBody, *shape.cpShape, *space.cpSpace)		; ChipmunkDestroyBody
		PostShapeFree (*shape, *space)
	EndProcedure
	ProcedureC cpPostBodyConstraintsFree (*body.cpBody, *constraint.cpConstraint, *space.cpSpace) ; ChipmunkDestroyBody
		PostConstraintFree (*constraint, *space)
	EndProcedure
	
	; Safe and future proof way to remove And free all objects that have been added To the space.
	; Don't forget to handle your local pointers to those obects! They will point to invalid adresses after this function.
	Procedure ChipmunkFreeSpaceChildren(*space.cpSpace)
		; How it works:
		; 1 step: attach Post... function to every object in space (code here)
		; 2 step: the Post function will be called by chipmunk and it adds callback to Free function for named object
		; 3 step: after space simulation step finished, the Free function is called and object is removed and freed
		
		;Must remove these BEFORE freeing the body Or you will access dangling pointers.
		cpSpaceEachShape(*space, @PostShapeFree(), *space)
	 	cpSpaceEachConstraint(*space, @PostConstraintFree(), *space)
	 	
	 	cpSpaceEachBody(*space, @PostBodyFree (), *space)
	EndProcedure

	; completely delete single body
	Procedure ChipmunkDestroyBody (*body.cpBody, *space.cpSpace)
		;Must remove these BEFORE freeing the body Or you will access dangling pointers.
		cpBodyEachShape(*body, @cpPostBodyShapesFree(), *space)
		cpBodyEachConstraint(*body, @cpPostBodyConstraintsFree(), *space)

		PostBodyFree (*body, *space)
	EndProcedure

	; outputs whole structure to debug window. let it be here for debug
	CompilerIf #PB_Compiler_Debugger
	Procedure XmlDbgRecurse (*Node, ChildID.l, svcRegExp.l)
		Protected CCount.l = XMLChildCount (*Node)
		Protected *NNode = NextXMLNode (*Node)
		Protected NType.l = XMLNodeType (*Node) 
		Protected Dim RegRes.s(0)
			If Not NType = #PB_XML_Root 
				ExtractRegularExpression(svcRegExp, GetXMLNodeText (*Node), RegRes())
					If CCount
						RegRes(0) = "\" + GetXMLNodeName (*Node) + ":"
					Else
		    			RegRes(0) = GetXMLNodeName (*Node) + " = " + RegRes(0)
		    		EndIf
	    		RegRes(0) = Space(GetXMLNodeOffset(*Node) - 4) + RegRes(0)
	    	Else ; root node = beginning
	    		RegRes(0) = "== [ STRUCTURE::" + Str(*Node) + " ] =="
	    	EndIf
				Debug RegRes(0)
	
			ChildID + 1
	     	If CCount >= ChildID : XmlDbgRecurse (ChildXMLNode (*Node), ChildID, svcRegExp) : EndIf
	     	If *NNode :	XmlDbgRecurse(*NNode, 0, svcRegExp) :  EndIf
	
		If NType = #PB_XML_Root : Debug "== [ END::" + Str(*Node) + " ] ==" : EndIf ; print memory adress at the end too, might be useful to ctrl+c
	EndProcedure
	CompilerEndIf
	; SYNTAX: StDebug (StructureVar, StructureType)
	Macro StDebug (DefinedStructure, DefinedType)
		CompilerIf #PB_Compiler_Debugger ; when debugger is off macro will remain to avoid error messages, but will do nothing
		  	Define tXML = CreateXML(#PB_Any)
		    	InsertXMLStructure(RootXMLNode(tXML), DefinedStructure, DefinedType)
		    	FormatXML(tXML, #PB_XML_ReFormat | #PB_XML_ReIndent | #PB_XML_CutNewline, 4)
			Define *RootNode = RootXMLNode (tXML)
			Define tEXP = CreateRegularExpression(#PB_Any, "(?s)([^ \f\n\r\t\v]|.*).*[^ \f\n\r\t\v]") ; what an "EXPRESSIVE POWER"
			    XmlDbgRecurse (*RootNode, 0, tEXP) ; had to move recurse procedure outside of macro, else macro will not work inside loops, etc
			FreeXML(tXML)
			FreeRegularExpression(tEXP)
		CompilerEndIf
	EndMacro

