﻿EnableExplicit
XIncludeFile "ChipMunk2D.pbi"

;==========================================================================================================================-
; This is another "helloword" by MadMedic
; It is some more complex, adding more objects and their visualization in 2D mode
;====================================================================================================-
InitSprite()
OpenWindow(0, 0, 0, 640, 480, "Chipmunk",  #PB_Window_SystemMenu | #PB_Window_ScreenCentered)
OpenWindowedScreen(WindowID(0), 0, 0, 640, 480)

Global  *space.cpSpace = cpSpaceNew() ; Create an empty space.
									  ;there is positive gravity - To safe screen coordinates orientation
Define gravity.cpVect : cpv(gravity, 0.0, 100) ; cpVect is a 2D vector And cpv() is a shortcut For initializing them.
cpSpaceSetGravity_pb(*space, gravity)	

; Add a Static line segment shape For the ground.
; We'll make it slightly tilted so the ball will roll off.
; We attach it To space->staticBody To tell Chipmunk it shouldn't be movable.
Define A.cpVect 
Define B.cpVect 
Define *ground.cpShape

;make borders
cpv(A, 0, 480) : cpv(B, 640, 480)
*ground.cpShape = cpSegmentShapeNew_pb(*space\staticBody, A, B, 0)
cpShapeSetFriction(*ground, 1)
cpSpaceAddShape(*space, *ground)

cpv(A, 640, 0) : cpv(B, 640, 480)
*ground.cpShape = cpSegmentShapeNew_pb(*space\staticBody, A, B, 0)
cpShapeSetFriction(*ground, 1)
cpSpaceAddShape(*space, *ground)

cpv(A, 0, 0) : cpv(B, 0, 480)
*ground.cpShape = cpSegmentShapeNew_pb(*space\staticBody, A, B, 0)
cpShapeSetFriction(*ground, 1)
cpSpaceAddShape(*space, *ground)


Structure BodyStruct
	Moment.cpFloat
	*Body.cpBody
	*Shape.cpShape
	pos.cpVect
	Sprite.l
	Width.l
	Height.l
EndStructure
Global NewList BodyList.BodyStruct()

Procedure Body_CircleAdd(x, y, radius, mass)
	AddElement(BodyList())
	BodyList()\Moment = cpMomentForCircle_pb(mass, 0, radius, cpvzero)
	BodyList()\Body = cpSpaceAddBody(*space, cpBodyNew(mass, BodyList()\moment))
	Protected BodyPos.cpVect : cpv(BodyPos, x, y)
	cpBodySetPosition_pb(BodyList()\Body, BodyPos)
	BodyList()\Shape = cpSpaceAddShape(*space, cpCircleShapeNew_pb(BodyList()\Body, radius, cpvzero))
	cpShapeSetFriction(BodyList()\Shape, 0.25)
	
	BodyList()\Width = radius * 2
	BodyList()\Height = radius * 2
	BodyList()\sprite = CreateSprite(#PB_Any, BodyList()\Width, BodyList()\Height, #PB_Sprite_AlphaBlending)
	StartDrawing(SpriteOutput(BodyList()\sprite))
	DrawingMode(#PB_2DDrawing_AllChannels)
	Box(0,0, BodyList()\Width, BodyList()\Height, RGBA(Random(255,155), Random(255,155), Random(255,155),0))
	DrawingMode(#PB_2DDrawing_Transparent)
	DrawText(BodyList()\Width * 0.5 - TextWidth("Chipmunk") * 0.5, BodyList()\Height * 0.5 - TextHeight("Chipmunk") * 0.5, "Chipmunk", RGB(0,0,0))
	DrawingMode(#PB_2DDrawing_AlphaChannel)
	Circle(BodyList()\Width *0.5, BodyList()\Height *0.5, BodyList()\Width*0.5, RGBA(255,255,255, 255))
	StopDrawing()
	
EndProcedure
Procedure Body_BoxAdd(x, y, width, height, mass)
	AddElement(BodyList())
	BodyList()\Moment = cpMomentForBox(mass, width, height)
	BodyList()\Body = cpSpaceAddBody(*space, cpBodyNew(mass, BodyList()\moment))
	Protected BodyPos.cpVect : cpv(BodyPos, x, y)
	cpBodySetPosition_pb(BodyList()\Body, BodyPos)
	BodyList()\Shape = cpSpaceAddShape(*space, cpBoxShapeNew(BodyList()\Body, width, height, 0))
	cpShapeSetFriction(BodyList()\Shape, 0.25)
	
	BodyList()\Width = width
	BodyList()\Height = height
	BodyList()\sprite = CreateSprite(#PB_Any, BodyList()\Width, BodyList()\Height, #PB_Sprite_AlphaBlending)
	StartDrawing(SpriteOutput(BodyList()\sprite))
	DrawingMode(#PB_2DDrawing_AllChannels)
	Box(0,0, BodyList()\Width, BodyList()\Height, RGBA(Random(255,155), Random(255,155), Random(255,155),255))
	DrawingMode(#PB_2DDrawing_Transparent)
	DrawText(BodyList()\Width * 0.5 - TextWidth("Chipmunk") * 0.5, BodyList()\Height * 0.5 - TextHeight("Chipmunk") * 0.5, "Chipmunk", RGB(0,0,0))
	StopDrawing()
	
EndProcedure
Procedure Body_Update()
	Protected pos.cpVect
	Protected Angle.cpFloat
	
	ForEach BodyList()
		cpBodyGetPosition_pb(BodyList()\Body, pos)
		Angle = Degree(cpBodyGetAngle(BodyList()\Body))
		RotateSprite(BodyList()\Sprite, Angle, #PB_Absolute)
		DisplayTransparentSprite(BodyList()\Sprite, pos\x - BodyList()\Width *0.5, pos\y - BodyList()\Height *0.5)
	Next
EndProcedure


Define i
For i=0 To 10
	Body_CircleAdd(Random(500,100), Random(200,100), Random(40,20), 1)
	Body_BoxAdd(Random(500,100), Random(200,100), Random(40,20), Random(40,20), 1)
Next 


; Now that it's all set up, we simulate all the objects in the space by
; stepping forward through time in small increments called steps.
; It is *highly* recommended To use a fixed size time Step.
Define timeStep.cpFloat = 1.0/60.0
Define Time.cpFloat

Define pos.cpVect, vel.cpVect
Define exit, event

Repeat
	; It's very important to process all the events remaining in the queue at each frame
	;
	exit = 0
	Repeat
		Event = WindowEvent()
		
		Select Event 
			Case #PB_Event_CloseWindow
				exit = 1
		EndSelect
	Until Event = 0
	
	If exit = 1
		Break
	EndIf
	
	cpSpaceStep(*space, timeStep)
	
	
	FlipBuffers() 
	ClearScreen(RGB(64, 128, 64))
	
	Body_Update()
	
	Delay(1)
ForEver


; Clean up our objects and exit!
; release sprites
ForEach BodyList()
	FreeSprite(BodyList()\Sprite)
Next

ChipmunkFreeSpaceChildren (*space) ; use this function to remove and destroy all space objects, instead of cleaning them separately
								   ; cpShapeFree(*ballShape)
								   ; cpBodyFree(*ballBody)
								   ; cpShapeFree(*ground)
cpSpaceFree(*space)

; IDE Options = PureBasic 5.40 LTS (Windows - x86)
; CursorPosition = 4
; Folding = --
; EnableUnicode
; EnableXP
; Executable = asdf.exe
; CPU = 1
; SubSystem = /OPENGL
; CompileSourceDirectory
; EnableCompileCount = 294
; EnableBuildCount = 1
; EnableExeConstant